"""
Python module generated from Java source file java.net.URLConnection

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.io import IOException
from java.io import InputStream
from java.io import OutputStream
from java.net import *
from java.security import AccessController
from java.security import Permission
from java.security import PrivilegedAction
from java.util import Collections
from java.util import Date
from java.util import Hashtable
from java.util import Iterator
from java.util import Locale
from java.util import Objects
from java.util import ServiceConfigurationError
from java.util import ServiceLoader
from java.util import StringTokenizer
from java.util.concurrent import ConcurrentHashMap
from sun.net.www import MessageHeader
from sun.security.action import GetPropertyAction
from sun.security.util import SecurityConstants
from typing import Any, Callable, Iterable, Tuple


class URLConnection:
    """
    The abstract class `URLConnection` is the superclass
    of all classes that represent a communications link between the
    application and a URL. Instances of this class can be used both to
    read from and to write to the resource referenced by the URL.
    
    
    In general, creating a connection to a URL is a multistep process:
    <ol>
    - The connection object is created by invoking the
        URL.openConnection() openConnection method on a URL.
    - The setup parameters and general request properties are manipulated.
    - The actual connection to the remote object is made, using the
       .connect() connect method.
    - The remote object becomes available. The header fields and the contents
        of the remote object can be accessed.
    </ol>
    
    The setup parameters are modified using the following methods:
    
      - `setAllowUserInteraction`
      - `setDoInput`
      - `setDoOutput`
      - `setIfModifiedSince`
      - `setUseCaches`
    
    
    and the general request properties are modified using the method:
    
      - `setRequestProperty`
    
    
    Default values for the `AllowUserInteraction` and
    `UseCaches` parameters can be set using the methods
    `setDefaultAllowUserInteraction` and
    `setDefaultUseCaches`.
    
    Each of the above `set` methods has a corresponding
    `get` method to retrieve the value of the parameter or
    general request property. The specific parameters and general
    request properties that are applicable are protocol specific.
    
    The following methods are used to access the header fields and
    the contents after the connection is made to the remote object:
    
      - `getContent`
      - `getHeaderField`
      - `getInputStream`
      - `getOutputStream`
    
    
    Certain header fields are accessed frequently. The methods:
    
      - `getContentEncoding`
      - `getContentLength`
      - `getContentType`
      - `getDate`
      - `getExpiration`
      - `getLastModified`
    
    
    provide convenient access to these fields. The
    `getContentType` method is used by the
    `getContent` method to determine the type of the remote
    object; subclasses may find it convenient to override the
    `getContentType` method.
    
    In the common case, all of the pre-connection parameters and
    general request properties can be ignored: the pre-connection
    parameters and request properties default to sensible values. For
    most clients of this interface, there are only two interesting
    methods: `getInputStream` and `getContent`,
    which are mirrored in the `URL` class by convenience methods.
    
    More information on the request properties and header fields of
    an `http` connection can be found at:
    <blockquote>```
    <a href="http://www.ietf.org/rfc/rfc2616.txt">http://www.ietf.org/rfc/rfc2616.txt</a>
    ```</blockquote>
    
    Invoking the `close()` methods on the `InputStream` or `OutputStream` of an
    `URLConnection` after a request may free network resources associated with this
    instance, unless particular protocol specifications specify different behaviours
    for it.

    Author(s)
    - James Gosling

    See
    - java.net.URLConnection.setUseCaches(boolean)

    Since
    - 1.0
    """

    @staticmethod
    def getFileNameMap() -> "FileNameMap":
        """
        Loads filename map (a mimetable) from a data file. It will
        first try to load the user-specific table, defined
        by &quot;content.types.user.table&quot; property. If that fails,
        it tries to load the default built-in table.

        Returns
        - the FileNameMap

        See
        - .setFileNameMap(java.net.FileNameMap)

        Since
        - 1.2
        """
        ...


    @staticmethod
    def setFileNameMap(map: "FileNameMap") -> None:
        """
        Sets the FileNameMap.
        
        If there is a security manager, this method first calls
        the security manager's `checkSetFactory` method
        to ensure the operation is allowed.
        This could result in a SecurityException.

        Arguments
        - map: the FileNameMap to be set

        Raises
        - SecurityException: if a security manager exists and its
                    `checkSetFactory` method doesn't allow the operation.

        See
        - .getFileNameMap()

        Since
        - 1.2
        """
        ...


    def connect(self) -> None:
        """
        Opens a communications link to the resource referenced by this
        URL, if such a connection has not already been established.
        
        If the `connect` method is called when the connection
        has already been opened (indicated by the `connected`
        field having the value `True`), the call is ignored.
        
        URLConnection objects go through two phases: first they are
        created, then they are connected.  After being created, and
        before being connected, various options can be specified
        (e.g., doInput and UseCaches).  After connecting, it is an
        error to try to set them.  Operations that depend on being
        connected, like getContentLength, will implicitly perform the
        connection, if necessary.

        Raises
        - SocketTimeoutException: if the timeout expires before
                      the connection can be established
        - IOException: if an I/O error occurs while opening the
                      connection.

        See
        - .setConnectTimeout(int)
        """
        ...


    def setConnectTimeout(self, timeout: int) -> None:
        """
        Sets a specified timeout value, in milliseconds, to be used
        when opening a communications link to the resource referenced
        by this URLConnection.  If the timeout expires before the
        connection can be established, a
        java.net.SocketTimeoutException is raised. A timeout of zero is
        interpreted as an infinite timeout.
        
         Some non-standard implementation of this method may ignore
        the specified timeout. To see the connect timeout set, please
        call getConnectTimeout().

        Arguments
        - timeout: an `int` that specifies the connect
                      timeout value in milliseconds

        Raises
        - IllegalArgumentException: if the timeout parameter is negative

        See
        - .connect()

        Since
        - 1.5
        """
        ...


    def getConnectTimeout(self) -> int:
        """
        Returns setting for connect timeout.
        
        0 return implies that the option is disabled
        (i.e., timeout of infinity).

        Returns
        - an `int` that indicates the connect timeout
                value in milliseconds

        See
        - .connect()

        Since
        - 1.5
        """
        ...


    def setReadTimeout(self, timeout: int) -> None:
        """
        Sets the read timeout to a specified timeout, in
        milliseconds. A non-zero value specifies the timeout when
        reading from Input stream when a connection is established to a
        resource. If the timeout expires before there is data available
        for read, a java.net.SocketTimeoutException is raised. A
        timeout of zero is interpreted as an infinite timeout.
        
         Some non-standard implementation of this method ignores the
        specified timeout. To see the read timeout set, please call
        getReadTimeout().

        Arguments
        - timeout: an `int` that specifies the timeout
        value to be used in milliseconds

        Raises
        - IllegalArgumentException: if the timeout parameter is negative

        See
        - InputStream.read()

        Since
        - 1.5
        """
        ...


    def getReadTimeout(self) -> int:
        """
        Returns setting for read timeout. 0 return implies that the
        option is disabled (i.e., timeout of infinity).

        Returns
        - an `int` that indicates the read timeout
                value in milliseconds

        See
        - InputStream.read()

        Since
        - 1.5
        """
        ...


    def getURL(self) -> "URL":
        """
        Returns the value of this `URLConnection`'s `URL`
        field.

        Returns
        - the value of this `URLConnection`'s `URL`
                 field.

        See
        - java.net.URLConnection.url
        """
        ...


    def getContentLength(self) -> int:
        """
        Returns the value of the `content-length` header field.
        <P>
        <B>Note</B>: .getContentLengthLong() getContentLengthLong()
        should be preferred over this method, since it returns a `long`
        instead and is therefore more portable.</P>

        Returns
        - the content length of the resource that this connection's URL
                 references, `-1` if the content length is not known,
                 or if the content length is greater than Integer.MAX_VALUE.
        """
        ...


    def getContentLengthLong(self) -> int:
        """
        Returns the value of the `content-length` header field as a
        long.

        Returns
        - the content length of the resource that this connection's URL
                 references, or `-1` if the content length is
                 not known.

        Since
        - 1.7
        """
        ...


    def getContentType(self) -> str:
        """
        Returns the value of the `content-type` header field.

        Returns
        - the content type of the resource that the URL references,
                 or `null` if not known.

        See
        - java.net.URLConnection.getHeaderField(java.lang.String)
        """
        ...


    def getContentEncoding(self) -> str:
        """
        Returns the value of the `content-encoding` header field.

        Returns
        - the content encoding of the resource that the URL references,
                 or `null` if not known.

        See
        - java.net.URLConnection.getHeaderField(java.lang.String)
        """
        ...


    def getExpiration(self) -> int:
        """
        Returns the value of the `expires` header field.

        Returns
        - the expiration date of the resource that this URL references,
                 or 0 if not known. The value is the number of milliseconds since
                 January 1, 1970 GMT.

        See
        - java.net.URLConnection.getHeaderField(java.lang.String)
        """
        ...


    def getDate(self) -> int:
        """
        Returns the value of the `date` header field.

        Returns
        - the sending date of the resource that the URL references,
                 or `0` if not known. The value returned is the
                 number of milliseconds since January 1, 1970 GMT.

        See
        - java.net.URLConnection.getHeaderField(java.lang.String)
        """
        ...


    def getLastModified(self) -> int:
        """
        Returns the value of the `last-modified` header field.
        The result is the number of milliseconds since January 1, 1970 GMT.

        Returns
        - the date the resource referenced by this
                 `URLConnection` was last modified, or 0 if not known.

        See
        - java.net.URLConnection.getHeaderField(java.lang.String)
        """
        ...


    def getHeaderField(self, name: str) -> str:
        """
        Returns the value of the named header field.
        
        If called on a connection that sets the same header multiple times
        with possibly different values, only the last value is returned.

        Arguments
        - name: the name of a header field.

        Returns
        - the value of the named header field, or `null`
                 if there is no such field in the header.
        """
        ...


    def getHeaderFields(self) -> dict[str, list[str]]:
        """
        Returns an unmodifiable Map of the header fields.
        The Map keys are Strings that represent the
        response-header field names. Each Map value is an
        unmodifiable List of Strings that represents
        the corresponding field values.

        Returns
        - a Map of header fields

        Since
        - 1.4
        """
        ...


    def getHeaderFieldInt(self, name: str, Default: int) -> int:
        """
        Returns the value of the named field parsed as a number.
        
        This form of `getHeaderField` exists because some
        connection types (e.g., `http-ng`) have pre-parsed
        headers. Classes for that connection type can override this method
        and short-circuit the parsing.

        Arguments
        - name: the name of the header field.
        - Default: the default value.

        Returns
        - the value of the named field, parsed as an integer. The
                 `Default` value is returned if the field is
                 missing or malformed.
        """
        ...


    def getHeaderFieldLong(self, name: str, Default: int) -> int:
        """
        Returns the value of the named field parsed as a number.
        
        This form of `getHeaderField` exists because some
        connection types (e.g., `http-ng`) have pre-parsed
        headers. Classes for that connection type can override this method
        and short-circuit the parsing.

        Arguments
        - name: the name of the header field.
        - Default: the default value.

        Returns
        - the value of the named field, parsed as a long. The
                 `Default` value is returned if the field is
                 missing or malformed.

        Since
        - 1.7
        """
        ...


    def getHeaderFieldDate(self, name: str, Default: int) -> int:
        """
        Returns the value of the named field parsed as date.
        The result is the number of milliseconds since January 1, 1970 GMT
        represented by the named field.
        
        This form of `getHeaderField` exists because some
        connection types (e.g., `http-ng`) have pre-parsed
        headers. Classes for that connection type can override this method
        and short-circuit the parsing.

        Arguments
        - name: the name of the header field.
        - Default: a default value.

        Returns
        - the value of the field, parsed as a date. The value of the
                 `Default` argument is returned if the field is
                 missing or malformed.
        """
        ...


    def getHeaderFieldKey(self, n: int) -> str:
        """
        Returns the key for the `n`<sup>th</sup> header field.
        Some implementations may treat the `0`<sup>th</sup>
        header field as special, in which case, .getHeaderField(int) getHeaderField(0)
        may return some value, but `getHeaderFieldKey(0)` returns `null`.
        For `n > 0` it returns `null` if there are fewer than `n+1` fields.

        Arguments
        - n: an index, where `n>=0`

        Returns
        - the key for the `n`<sup>th</sup> header field,
                 or `null` if there are fewer than `n+1`
                 fields when `n > 0`.
        """
        ...


    def getHeaderField(self, n: int) -> str:
        """
        Returns the value for the `n`<sup>th</sup> header field.
        It returns `null` if there are fewer than
        `n+1` fields.
        
        This method can be used in conjunction with the
        .getHeaderFieldKey(int) getHeaderFieldKey method to iterate through all
        the headers in the message.

        Arguments
        - n: an index, where `n>=0`

        Returns
        - the value of the `n`<sup>th</sup> header field
                 or `null` if there are fewer than `n+1` fields

        See
        - java.net.URLConnection.getHeaderFieldKey(int)
        """
        ...


    def getContent(self) -> "Object":
        """
        Retrieves the contents of this URL connection.
        
        This method first determines the content type of the object by
        calling the `getContentType` method. If this is
        the first time that the application has seen that specific content
        type, a content handler for that content type is created.
         This is done as follows:
        <ol>
        - If the application has set up a content handler factory instance
            using the `setContentHandlerFactory` method, the
            `createContentHandler` method of that instance is called
            with the content type as an argument; the result is a content
            handler for that content type.
        - If no `ContentHandlerFactory` has yet been set up,
            or if the factory's `createContentHandler` method
            returns `null`, then the java.util.ServiceLoader
            ServiceLoader mechanism is used to locate java.net.ContentHandlerFactory ContentHandlerFactory
            implementations using the system class
            loader. The order that factories are located is implementation
            specific, and an implementation is free to cache the located
            factories. A java.util.ServiceConfigurationError
            ServiceConfigurationError, `Error` or `RuntimeException`
            thrown from the `createContentHandler`, if encountered, will
            be propagated to the calling thread. The `createContentHandler` method of each factory, if instantiated, is
            invoked, with the content type, until a factory returns non-null,
            or all factories have been exhausted.
        - Failing that, this method tries to load a content handler
            class as defined by java.net.ContentHandler ContentHandler.
            If the class does not exist, or is not a subclass of `ContentHandler`, then an `UnknownServiceException` is thrown.
        </ol>

        Returns
        - the object fetched. The `instanceof` operator
                      should be used to determine the specific kind of object
                      returned.

        Raises
        - IOException: if an I/O error occurs while
                      getting the content.
        - UnknownServiceException: if the protocol does not support
                      the content type.

        See
        - java.net.URLConnection.setContentHandlerFactory(java.net.ContentHandlerFactory)
        """
        ...


    def getContent(self, classes: list[type[Any]]) -> "Object":
        """
        Retrieves the contents of this URL connection.

        Arguments
        - classes: the `Class` array
        indicating the requested types

        Returns
        - the object fetched that is the first match of the type
                      specified in the classes array. null if none of
                      the requested types are supported.
                      The `instanceof` operator should be used to
                      determine the specific kind of object returned.

        Raises
        - IOException: if an I/O error occurs while
                      getting the content.
        - UnknownServiceException: if the protocol does not support
                      the content type.

        See
        - java.net.URLConnection.setContentHandlerFactory(java.net.ContentHandlerFactory)

        Since
        - 1.3
        """
        ...


    def getPermission(self) -> "Permission":
        """
        Returns a permission object representing the permission
        necessary to make the connection represented by this
        object. This method returns null if no permission is
        required to make the connection. By default, this method
        returns `java.security.AllPermission`. Subclasses
        should override this method and return the permission
        that best represents the permission required to make
        a connection to the URL. For example, a `URLConnection`
        representing a `file:` URL would return a
        `java.io.FilePermission` object.
        
        The permission returned may dependent upon the state of the
        connection. For example, the permission before connecting may be
        different from that after connecting. For example, an HTTP
        sever, say foo.com, may redirect the connection to a different
        host, say bar.com. Before connecting the permission returned by
        the connection will represent the permission needed to connect
        to foo.com, while the permission returned after connecting will
        be to bar.com.
        
        Permissions are generally used for two purposes: to protect
        caches of objects obtained through URLConnections, and to check
        the right of a recipient to learn about a particular URL. In
        the first case, the permission should be obtained
        *after* the object has been obtained. For example, in an
        HTTP connection, this will represent the permission to connect
        to the host from which the data was ultimately fetched. In the
        second case, the permission should be obtained and tested
        *before* connecting.

        Returns
        - the permission object representing the permission
        necessary to make the connection represented by this
        URLConnection.

        Raises
        - IOException: if the computation of the permission
        requires network or file I/O and an exception occurs while
        computing it.
        """
        ...


    def getInputStream(self) -> "InputStream":
        """
        Returns an input stream that reads from this open connection.
        
        A SocketTimeoutException can be thrown when reading from the
        returned input stream if the read timeout expires before data
        is available for read.

        Returns
        - an input stream that reads from this open connection.

        Raises
        - IOException: if an I/O error occurs while
                      creating the input stream.
        - UnknownServiceException: if the protocol does not support
                      input.

        See
        - .getReadTimeout()
        """
        ...


    def getOutputStream(self) -> "OutputStream":
        """
        Returns an output stream that writes to this connection.

        Returns
        - an output stream that writes to this connection.

        Raises
        - IOException: if an I/O error occurs while
                      creating the output stream.
        - UnknownServiceException: if the protocol does not support
                      output.
        """
        ...


    def toString(self) -> str:
        """
        Returns a `String` representation of this URL connection.

        Returns
        - a string representation of this `URLConnection`.
        """
        ...


    def setDoInput(self, doinput: bool) -> None:
        """
        Sets the value of the `doInput` field for this
        `URLConnection` to the specified value.
        
        A URL connection can be used for input and/or output.  Set the doInput
        flag to True if you intend to use the URL connection for input,
        False if not.  The default is True.

        Arguments
        - doinput: the new value.

        Raises
        - IllegalStateException: if already connected

        See
        - .getDoInput()
        """
        ...


    def getDoInput(self) -> bool:
        """
        Returns the value of this `URLConnection`'s
        `doInput` flag.

        Returns
        - the value of this `URLConnection`'s
                 `doInput` flag.

        See
        - .setDoInput(boolean)
        """
        ...


    def setDoOutput(self, dooutput: bool) -> None:
        """
        Sets the value of the `doOutput` field for this
        `URLConnection` to the specified value.
        
        A URL connection can be used for input and/or output.  Set the doOutput
        flag to True if you intend to use the URL connection for output,
        False if not.  The default is False.

        Arguments
        - dooutput: the new value.

        Raises
        - IllegalStateException: if already connected

        See
        - .getDoOutput()
        """
        ...


    def getDoOutput(self) -> bool:
        """
        Returns the value of this `URLConnection`'s
        `doOutput` flag.

        Returns
        - the value of this `URLConnection`'s
                 `doOutput` flag.

        See
        - .setDoOutput(boolean)
        """
        ...


    def setAllowUserInteraction(self, allowuserinteraction: bool) -> None:
        """
        Set the value of the `allowUserInteraction` field of
        this `URLConnection`.

        Arguments
        - allowuserinteraction: the new value.

        Raises
        - IllegalStateException: if already connected

        See
        - .getAllowUserInteraction()
        """
        ...


    def getAllowUserInteraction(self) -> bool:
        """
        Returns the value of the `allowUserInteraction` field for
        this object.

        Returns
        - the value of the `allowUserInteraction` field for
                 this object.

        See
        - .setAllowUserInteraction(boolean)
        """
        ...


    @staticmethod
    def setDefaultAllowUserInteraction(defaultallowuserinteraction: bool) -> None:
        """
        Sets the default value of the
        `allowUserInteraction` field for all future
        `URLConnection` objects to the specified value.

        Arguments
        - defaultallowuserinteraction: the new value.

        See
        - .getDefaultAllowUserInteraction()
        """
        ...


    @staticmethod
    def getDefaultAllowUserInteraction() -> bool:
        """
        Returns the default value of the `allowUserInteraction`
        field.
        
        This default is "sticky", being a part of the static state of all
        URLConnections.  This flag applies to the next, and all following
        URLConnections that are created.

        Returns
        - the default value of the `allowUserInteraction`
                 field.

        See
        - .setDefaultAllowUserInteraction(boolean)
        """
        ...


    def setUseCaches(self, usecaches: bool) -> None:
        """
        Sets the value of the `useCaches` field of this
        `URLConnection` to the specified value.
        
        Some protocols do caching of documents.  Occasionally, it is important
        to be able to "tunnel through" and ignore the caches (e.g., the
        "reload" button in a browser).  If the UseCaches flag on a connection
        is True, the connection is allowed to use whatever caches it can.
         If False, caches are to be ignored.
         The default value comes from defaultUseCaches, which defaults to
        True. A default value can also be set per-protocol using
        .setDefaultUseCaches(String,boolean).

        Arguments
        - usecaches: a `boolean` indicating whether
        or not to allow caching

        Raises
        - IllegalStateException: if already connected

        See
        - .getUseCaches()
        """
        ...


    def getUseCaches(self) -> bool:
        """
        Returns the value of this `URLConnection`'s
        `useCaches` field.

        Returns
        - the value of this `URLConnection`'s
                 `useCaches` field.

        See
        - .setUseCaches(boolean)
        """
        ...


    def setIfModifiedSince(self, ifmodifiedsince: int) -> None:
        """
        Sets the value of the `ifModifiedSince` field of
        this `URLConnection` to the specified value.

        Arguments
        - ifmodifiedsince: the new value.

        Raises
        - IllegalStateException: if already connected

        See
        - .getIfModifiedSince()
        """
        ...


    def getIfModifiedSince(self) -> int:
        """
        Returns the value of this object's `ifModifiedSince` field.

        Returns
        - the value of this object's `ifModifiedSince` field.

        See
        - .setIfModifiedSince(long)
        """
        ...


    def getDefaultUseCaches(self) -> bool:
        """
        Returns the default value of a `URLConnection`'s
        `useCaches` flag.
        
        This default is "sticky", being a part of the static state of all
        URLConnections.  This flag applies to the next, and all following
        URLConnections that are created. This default value can be over-ridden
        per protocol using .setDefaultUseCaches(String,boolean)

        Returns
        - the default value of a `URLConnection`'s
                 `useCaches` flag.

        See
        - .setDefaultUseCaches(boolean)
        """
        ...


    def setDefaultUseCaches(self, defaultusecaches: bool) -> None:
        """
        Sets the default value of the `useCaches` field to the
        specified value. This default value can be over-ridden
        per protocol using .setDefaultUseCaches(String,boolean)

        Arguments
        - defaultusecaches: the new value.

        See
        - .getDefaultUseCaches()
        """
        ...


    @staticmethod
    def setDefaultUseCaches(protocol: str, defaultVal: bool) -> None:
        """
        Sets the default value of the `useCaches` field for the named
        protocol to the given value. This value overrides any default setting
        set by .setDefaultUseCaches(boolean) for the given protocol.
        Successive calls to this method change the setting and affect the
        default value for all future connections of that protocol. The protocol
        name is case insensitive.

        Arguments
        - protocol: the protocol to set the default for
        - defaultVal: whether caching is enabled by default for the given protocol

        Since
        - 9
        """
        ...


    @staticmethod
    def getDefaultUseCaches(protocol: str) -> bool:
        """
        Returns the default value of the `useCaches` flag for the given protocol. If
        .setDefaultUseCaches(String,boolean) was called for the given protocol,
        then that value is returned. Otherwise, if .setDefaultUseCaches(boolean)
        was called, then that value is returned. If neither method was called,
        the return value is `True`. The protocol name is case insensitive.

        Arguments
        - protocol: the protocol whose defaultUseCaches setting is required

        Returns
        - the default value of the `useCaches` flag for the given protocol.

        Since
        - 9
        """
        ...


    def setRequestProperty(self, key: str, value: str) -> None:
        """
        Sets the general request property. If a property with the key already
        exists, overwrite its value with the new value.
        
         NOTE: HTTP requires all request properties which can
        legally have multiple instances with the same key
        to use a comma-separated list syntax which enables multiple
        properties to be appended into a single property.

        Arguments
        - key: the keyword by which the request is known
                         (e.g., "`Accept`").
        - value: the value associated with it.

        Raises
        - IllegalStateException: if already connected
        - NullPointerException: if key is `null`

        See
        - .getRequestProperty(java.lang.String)
        """
        ...


    def addRequestProperty(self, key: str, value: str) -> None:
        """
        Adds a general request property specified by a
        key-value pair.  This method will not overwrite
        existing values associated with the same key.

        Arguments
        - key: the keyword by which the request is known
                         (e.g., "`Accept`").
        - value: the value associated with it.

        Raises
        - IllegalStateException: if already connected
        - NullPointerException: if key is null

        See
        - .getRequestProperties()

        Since
        - 1.4
        """
        ...


    def getRequestProperty(self, key: str) -> str:
        """
        Returns the value of the named general request property for this
        connection.

        Arguments
        - key: the keyword by which the request is known (e.g., "Accept").

        Returns
        - the value of the named general request property for this
                  connection. If key is null, then null is returned.

        Raises
        - IllegalStateException: if already connected

        See
        - .setRequestProperty(java.lang.String, java.lang.String)
        """
        ...


    def getRequestProperties(self) -> dict[str, list[str]]:
        """
        Returns an unmodifiable Map of general request
        properties for this connection. The Map keys
        are Strings that represent the request-header
        field names. Each Map value is a unmodifiable List
        of Strings that represents the corresponding
        field values.

        Returns
        - a Map of the general request properties for this connection.

        Raises
        - IllegalStateException: if already connected

        Since
        - 1.4
        """
        ...


    @staticmethod
    def setDefaultRequestProperty(key: str, value: str) -> None:
        """
        Sets the default value of a general request property. When a
        `URLConnection` is created, it is initialized with
        these properties.

        Arguments
        - key: the keyword by which the request is known
                         (e.g., "`Accept`").
        - value: the value associated with the key.

        See
        - .getDefaultRequestProperty(java.lang.String)

        Deprecated
        - The instance specific setRequestProperty method
        should be used after an appropriate instance of URLConnection
        is obtained. Invoking this method will have no effect.
        """
        ...


    @staticmethod
    def getDefaultRequestProperty(key: str) -> str:
        """
        Returns the value of the default request property. Default request
        properties are set for every connection.

        Arguments
        - key: the keyword by which the request is known (e.g., "Accept").

        Returns
        - the value of the default request property
        for the specified key.

        See
        - .setDefaultRequestProperty(java.lang.String, java.lang.String)

        Deprecated
        - The instance specific getRequestProperty method
        should be used after an appropriate instance of URLConnection
        is obtained.
        """
        ...


    @staticmethod
    def setContentHandlerFactory(fac: "ContentHandlerFactory") -> None:
        """
        Sets the `ContentHandlerFactory` of an
        application. It can be called at most once by an application.
        
        The `ContentHandlerFactory` instance is used to
        construct a content handler from a content type.
        
        If there is a security manager, this method first calls
        the security manager's `checkSetFactory` method
        to ensure the operation is allowed.
        This could result in a SecurityException.

        Arguments
        - fac: the desired factory.

        Raises
        - Error: if the factory has already been defined.
        - SecurityException: if a security manager exists and its
                    `checkSetFactory` method doesn't allow the operation.

        See
        - SecurityManager.checkSetFactory
        """
        ...


    @staticmethod
    def guessContentTypeFromName(fname: str) -> str:
        """
        Tries to determine the content type of an object, based
        on the specified "file" component of a URL.
        This is a convenience method that can be used by
        subclasses that override the `getContentType` method.

        Arguments
        - fname: a filename.

        Returns
        - a guess as to what the content type of the object is,
                 based upon its file name.

        See
        - java.net.URLConnection.getContentType()
        """
        ...


    @staticmethod
    def guessContentTypeFromStream(is: "InputStream") -> str:
        """
        Tries to determine the type of an input stream based on the
        characters at the beginning of the input stream. This method can
        be used by subclasses that override the
        `getContentType` method.
        
        Ideally, this routine would not be needed. But many
        `http` servers return the incorrect content type; in
        addition, there are many nonstandard extensions. Direct inspection
        of the bytes to determine the content type is often more accurate
        than believing the content type claimed by the `http` server.

        Arguments
        - is: an input stream that supports marks.

        Returns
        - a guess at the content type, or `null` if none
                    can be determined.

        Raises
        - IOException: if an I/O error occurs while reading the
                      input stream.

        See
        - java.net.URLConnection.getContentType()
        """
        ...



"""
Python module generated from Java source file java.io.FilterInputStream

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.io import *
from typing import Any, Callable, Iterable, Tuple


class FilterInputStream(InputStream):
    """
    A `FilterInputStream` contains
    some other input stream, which it uses as
    its  basic source of data, possibly transforming
    the data along the way or providing  additional
    functionality. The class `FilterInputStream`
    itself simply overrides all  methods of
    `InputStream` with versions that
    pass all requests to the contained  input
    stream. Subclasses of `FilterInputStream`
    may further override some of  these methods
    and may also provide additional methods
    and fields.

    Author(s)
    - Jonathan Payne

    Since
    - 1.0
    """

    def read(self) -> int:
        """
        Reads the next byte of data from this input stream. The value
        byte is returned as an `int` in the range
        `0` to `255`. If no byte is available
        because the end of the stream has been reached, the value
        `-1` is returned. This method blocks until input data
        is available, the end of the stream is detected, or an exception
        is thrown.
        
        This method
        simply performs `in.read()` and returns the result.

        Returns
        - the next byte of data, or `-1` if the end of the
                    stream is reached.

        Raises
        - IOException: if an I/O error occurs.

        See
        - java.io.FilterInputStream.in
        """
        ...


    def read(self, b: list[int]) -> int:
        """
        Reads up to `b.length` bytes of data from this
        input stream into an array of bytes. This method blocks until some
        input is available.
        
        This method simply performs the call
        `read(b, 0, b.length)` and returns
        the  result. It is important that it does
        *not* do `in.read(b)` instead;
        certain subclasses of  `FilterInputStream`
        depend on the implementation strategy actually
        used.

        Arguments
        - b: the buffer into which the data is read.

        Returns
        - the total number of bytes read into the buffer, or
                    `-1` if there is no more data because the end of
                    the stream has been reached.

        Raises
        - IOException: if an I/O error occurs.

        See
        - java.io.FilterInputStream.read(byte[], int, int)
        """
        ...


    def read(self, b: list[int], off: int, len: int) -> int:
        """
        Reads up to `len` bytes of data from this input stream
        into an array of bytes. If `len` is not zero, the method
        blocks until some input is available; otherwise, no
        bytes are read and `0` is returned.
        
        This method simply performs `in.read(b, off, len)`
        and returns the result.

        Arguments
        - b: the buffer into which the data is read.
        - off: the start offset in the destination array `b`
        - len: the maximum number of bytes read.

        Returns
        - the total number of bytes read into the buffer, or
                    `-1` if there is no more data because the end of
                    the stream has been reached.

        Raises
        - NullPointerException: If `b` is `null`.
        - IndexOutOfBoundsException: If `off` is negative,
                    `len` is negative, or `len` is greater than
                    `b.length - off`
        - IOException: if an I/O error occurs.

        See
        - java.io.FilterInputStream.in
        """
        ...


    def skip(self, n: int) -> int:
        """
        Skips over and discards `n` bytes of data from the
        input stream. The `skip` method may, for a variety of
        reasons, end up skipping over some smaller number of bytes,
        possibly `0`. The actual number of bytes skipped is
        returned.
        
        This method simply performs `in.skip(n)`.

        Arguments
        - n: the number of bytes to be skipped.

        Returns
        - the actual number of bytes skipped.

        Raises
        - IOException: if `in.skip(n)` throws an IOException.
        """
        ...


    def available(self) -> int:
        """
        Returns an estimate of the number of bytes that can be read (or
        skipped over) from this input stream without blocking by the next
        caller of a method for this input stream. The next caller might be
        the same thread or another thread.  A single read or skip of this
        many bytes will not block, but may read or skip fewer bytes.
        
        This method returns the result of .in in.available().

        Returns
        - an estimate of the number of bytes that can be read (or skipped
                    over) from this input stream without blocking.

        Raises
        - IOException: if an I/O error occurs.
        """
        ...


    def close(self) -> None:
        """
        Closes this input stream and releases any system resources
        associated with the stream.
        This
        method simply performs `in.close()`.

        Raises
        - IOException: if an I/O error occurs.

        See
        - java.io.FilterInputStream.in
        """
        ...


    def mark(self, readlimit: int) -> None:
        """
        Marks the current position in this input stream. A subsequent
        call to the `reset` method repositions this stream at
        the last marked position so that subsequent reads re-read the same bytes.
        
        The `readlimit` argument tells this input stream to
        allow that many bytes to be read before the mark position gets
        invalidated.
        
        This method simply performs `in.mark(readlimit)`.

        Arguments
        - readlimit: the maximum limit of bytes that can be read before
                             the mark position becomes invalid.

        See
        - java.io.FilterInputStream.reset()
        """
        ...


    def reset(self) -> None:
        """
        Repositions this stream to the position at the time the
        `mark` method was last called on this input stream.
        
        This method
        simply performs `in.reset()`.
        
        Stream marks are intended to be used in
        situations where you need to read ahead a little to see what's in
        the stream. Often this is most easily done by invoking some
        general parser. If the stream is of the type handled by the
        parse, it just chugs along happily. If the stream is not of
        that type, the parser should toss an exception when it fails.
        If this happens within readlimit bytes, it allows the outer
        code to reset the stream and try another parser.

        Raises
        - IOException: if the stream has not been marked or if the
                      mark has been invalidated.

        See
        - java.io.FilterInputStream.mark(int)
        """
        ...


    def markSupported(self) -> bool:
        """
        Tests if this input stream supports the `mark`
        and `reset` methods.
        This method
        simply performs `in.markSupported()`.

        Returns
        - `True` if this stream type supports the
                 `mark` and `reset` method;
                 `False` otherwise.

        See
        - java.io.InputStream.reset()
        """
        ...



"""
Python module generated from Java source file com.google.common.collect.GwtTransient

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.collect import *
from typing import Any, Callable, Iterable, Tuple


class GwtTransient:
    """
    Private replacement for com.google.gwt.user.client.rpc.GwtTransient to work around
    build-system quirks. This annotation should be used **only** in `com.google.common.collect`.
    """





"""
Python module generated from Java source file org.bukkit.conversations.MessagePrompt

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.conversations import *
from typing import Any, Callable, Iterable, Tuple


class MessagePrompt(Prompt):
    """
    MessagePrompt is the base class for any prompt that only displays a message
    to the user and requires no input.
    """

    def __init__(self):
        ...


    def blocksForInput(self, context: "ConversationContext") -> bool:
        """
        Message prompts never wait for user input before continuing.

        Arguments
        - context: Context information about the conversation.

        Returns
        - Always False.
        """
        ...


    def acceptInput(self, context: "ConversationContext", input: str) -> "Prompt":
        """
        Accepts and ignores any user input, returning the next prompt in the
        prompt graph instead.

        Arguments
        - context: Context information about the conversation.
        - input: Ignored.

        Returns
        - The next prompt in the prompt graph.
        """
        ...



"""
Python module generated from Java source file org.bukkit.entity.AreaEffectCloud

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit import Color
from org.bukkit import Particle
from org.bukkit.entity import *
from org.bukkit.potion import PotionData
from org.bukkit.potion import PotionEffect
from org.bukkit.potion import PotionEffectType
from org.bukkit.potion import PotionType
from org.bukkit.projectiles import ProjectileSource
from typing import Any, Callable, Iterable, Tuple


class AreaEffectCloud(Entity):
    """
    Represents an area effect cloud which will imbue a potion effect onto
    entities which enter it.
    """

    def getDuration(self) -> int:
        """
        Gets the duration which this cloud will exist for (in ticks).

        Returns
        - cloud duration
        """
        ...


    def setDuration(self, duration: int) -> None:
        """
        Sets the duration which this cloud will exist for (in ticks).

        Arguments
        - duration: cloud duration
        """
        ...


    def getWaitTime(self) -> int:
        """
        Gets the time which an entity has to be exposed to the cloud before the
        effect is applied.

        Returns
        - wait time
        """
        ...


    def setWaitTime(self, waitTime: int) -> None:
        """
        Sets the time which an entity has to be exposed to the cloud before the
        effect is applied.

        Arguments
        - waitTime: wait time
        """
        ...


    def getReapplicationDelay(self) -> int:
        """
        Gets the time that an entity will be immune from subsequent exposure.

        Returns
        - reapplication delay
        """
        ...


    def setReapplicationDelay(self, delay: int) -> None:
        """
        Sets the time that an entity will be immune from subsequent exposure.

        Arguments
        - delay: reapplication delay
        """
        ...


    def getDurationOnUse(self) -> int:
        """
        Gets the amount that the duration of this cloud will decrease by when it
        applies an effect to an entity.

        Returns
        - duration on use delta
        """
        ...


    def setDurationOnUse(self, duration: int) -> None:
        """
        Sets the amount that the duration of this cloud will decrease by when it
        applies an effect to an entity.

        Arguments
        - duration: duration on use delta
        """
        ...


    def getRadius(self) -> float:
        """
        Gets the initial radius of the cloud.

        Returns
        - radius
        """
        ...


    def setRadius(self, radius: float) -> None:
        """
        Sets the initial radius of the cloud.

        Arguments
        - radius: radius
        """
        ...


    def getRadiusOnUse(self) -> float:
        """
        Gets the amount that the radius of this cloud will decrease by when it
        applies an effect to an entity.

        Returns
        - radius on use delta
        """
        ...


    def setRadiusOnUse(self, radius: float) -> None:
        """
        Sets the amount that the radius of this cloud will decrease by when it
        applies an effect to an entity.

        Arguments
        - radius: radius on use delta
        """
        ...


    def getRadiusPerTick(self) -> float:
        """
        Gets the amount that the radius of this cloud will decrease by each tick.

        Returns
        - radius per tick delta
        """
        ...


    def setRadiusPerTick(self, radius: float) -> None:
        """
        Gets the amount that the radius of this cloud will decrease by each tick.

        Arguments
        - radius: per tick delta
        """
        ...


    def getParticle(self) -> "Particle":
        """
        Gets the particle which this cloud will be composed of

        Returns
        - particle the set particle type
        """
        ...


    def setParticle(self, particle: "Particle") -> None:
        """
        Sets the particle which this cloud will be composed of

        Arguments
        - particle: the new particle type
        """
        ...


    def setParticle(self, particle: "Particle", data: "T") -> None:
        """
        Sets the particle which this cloud will be composed of
        
        Type `<T>`: type of particle data (see Particle.getDataType()

        Arguments
        - particle: the new particle type
        - data: the data to use for the particle or null,
                    the type of this depends on Particle.getDataType()
        """
        ...


    def setBasePotionData(self, data: "PotionData") -> None:
        """
        Sets the underlying potion data

        Arguments
        - data: PotionData to set the base potion state to

        Deprecated
        - Upgraded / extended potions are now their own PotionType use .setBasePotionType instead.
        """
        ...


    def getBasePotionData(self) -> "PotionData":
        """
        Returns the potion data about the base potion

        Returns
        - a PotionData object

        Deprecated
        - Upgraded / extended potions are now their own PotionType use .getBasePotionType() instead.
        """
        ...


    def setBasePotionType(self, type: "PotionType") -> None:
        """
        Sets the underlying potion type

        Arguments
        - type: PotionType to set the base potion state to
        """
        ...


    def getBasePotionType(self) -> "PotionType":
        """
        Returns the potion type about the base potion

        Returns
        - a PotionType object
        """
        ...


    def hasCustomEffects(self) -> bool:
        """
        Checks for the presence of custom potion effects.

        Returns
        - True if custom potion effects are applied
        """
        ...


    def getCustomEffects(self) -> list["PotionEffect"]:
        """
        Gets an immutable list containing all custom potion effects applied to
        this cloud.
        
        Plugins should check that hasCustomEffects() returns True before calling
        this method.

        Returns
        - the immutable list of custom potion effects
        """
        ...


    def addCustomEffect(self, effect: "PotionEffect", overwrite: bool) -> bool:
        """
        Adds a custom potion effect to this cloud.

        Arguments
        - effect: the potion effect to add
        - overwrite: True if any existing effect of the same type should be
        overwritten

        Returns
        - True if the effect was added as a result of this call
        """
        ...


    def removeCustomEffect(self, type: "PotionEffectType") -> bool:
        """
        Removes a custom potion effect from this cloud.

        Arguments
        - type: the potion effect type to remove

        Returns
        - True if the an effect was removed as a result of this call
        """
        ...


    def hasCustomEffect(self, type: "PotionEffectType") -> bool:
        """
        Checks for a specific custom potion effect type on this cloud.

        Arguments
        - type: the potion effect type to check for

        Returns
        - True if the potion has this effect
        """
        ...


    def clearCustomEffects(self) -> None:
        """
        Removes all custom potion effects from this cloud.
        """
        ...


    def getColor(self) -> "Color":
        """
        Gets the color of this cloud. Will be applied as a tint to its particles.

        Returns
        - cloud color
        """
        ...


    def setColor(self, color: "Color") -> None:
        """
        Sets the color of this cloud. Will be applied as a tint to its particles.

        Arguments
        - color: cloud color
        """
        ...


    def getSource(self) -> "ProjectileSource":
        """
        Retrieve the original source of this cloud.

        Returns
        - the ProjectileSource that threw the LingeringPotion
        """
        ...


    def setSource(self, source: "ProjectileSource") -> None:
        """
        Set the original source of this cloud.

        Arguments
        - source: the ProjectileSource that threw the LingeringPotion
        """
        ...



"""
Python module generated from Java source file org.bukkit.inventory.CreativeCategory

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from enum import Enum
from org.bukkit.inventory import *
from typing import Any, Callable, Iterable, Tuple


class CreativeCategory(Enum):
    """
    Represents a category in the creative inventory.
    """

    BUILDING_BLOCKS = 0
    """
    An assortment of building blocks including dirt, bricks, planks, ores
    slabs, etc.
    """
    DECORATIONS = 1
    """
    Blocks and items typically used for decorative purposes including
    candles, saplings, flora, fauna, fences, walls, carpets, etc.
    """
    REDSTONE = 2
    """
    Blocks used and associated with redstone contraptions including buttons,
    levers, pressure plates, redstone components, pistons, etc.
    """
    TRANSPORTATION = 3
    """
    Items pertaining to transportation including minecarts, rails, boats,
    elytra, etc.
    """
    MISC = 4
    """
    Miscellaneous items and blocks that do not fit into other categories
    including gems, dyes, spawn eggs, discs, banner patterns, etc.
    """
    FOOD = 5
    """
    Food items consumable by the player including meats, berries, edible
    drops from creatures, etc.
    """
    TOOLS = 6
    """
    Equipment items meant for general utility including pickaxes, axes, hoes,
    flint and steel, and useful enchantment books for said tools.
    """
    COMBAT = 7
    """
    Equipment items meant for combat including armor, swords, bows, tipped
    arrows, and useful enchantment books for said equipment.
    """
    BREWING = 8
    """
    All items related to brewing and potions including all types of potions,
    their variants, and ingredients to brew them.
    """



"""
Python module generated from Java source file com.google.common.primitives.ShortsMethodsForWeb

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.primitives import *
from typing import Any, Callable, Iterable, Tuple


class ShortsMethodsForWeb:
    """
    Holder for web specializations of methods of `Shorts`. Intended to be empty for regular
    version.
    """





"""
Python module generated from Java source file com.google.common.collect.TransformedListIterator

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.base import Function
from com.google.common.collect import *
from java.util import ListIterator
from org.checkerframework.checker.nullness.qual import Nullable
from typing import Any, Callable, Iterable, Tuple


class TransformedListIterator(TransformedIterator, ListIterator):
    """
    An iterator that transforms a backing list iterator; for internal use. This avoids the object
    overhead of constructing a Function for internal methods.

    Author(s)
    - Louis Wasserman
    """

    def hasPrevious(self) -> bool:
        ...


    def previous(self) -> "T":
        ...


    def nextIndex(self) -> int:
        ...


    def previousIndex(self) -> int:
        ...


    def set(self, element: "T") -> None:
        ...


    def add(self, element: "T") -> None:
        ...



"""
Python module generated from Java source file org.bukkit.packs.ResourcePack

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.util import UUID
from org.bukkit.packs import *
from typing import Any, Callable, Iterable, Tuple


class ResourcePack:
    """
    Represents a resource pack.

    See
    - <a href="https://minecraft.wiki/w/Resource_pack">Minecraft wiki</a>
    """

    def getId(self) -> "UUID":
        """
        Gets the id of the resource pack.

        Returns
        - the id
        """
        ...


    def getUrl(self) -> str:
        """
        Gets the url of the resource pack.

        Returns
        - the url
        """
        ...


    def getHash(self) -> str:
        """
        Gets the hash of the resource pack.

        Returns
        - the hash
        """
        ...


    def getPrompt(self) -> str:
        """
        Gets the prompt to show of the resource pack.

        Returns
        - the prompt
        """
        ...


    def isRequired(self) -> bool:
        """
        Gets if the resource pack is required by the server.

        Returns
        - True if is required
        """
        ...



"""
Python module generated from Java source file com.google.common.collect.RangeSet

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtIncompatible
from com.google.common.collect import *
from com.google.errorprone.annotations import DoNotMock
from java.util import NoSuchElementException
from javax.annotation import CheckForNull
from typing import Any, Callable, Iterable, Tuple


class RangeSet:
    """
    A set comprising zero or more Range.isEmpty nonempty, Range.isConnected(Range) disconnected ranges of type `C`.
    
    Implementations that choose to support the .add(Range) operation are required to
    ignore empty ranges and coalesce connected ranges. For example:
    
    ````RangeSet<Integer> rangeSet = TreeRangeSet.create();
    rangeSet.add(Range.closed(1, 10)); // {[1, 10]`
    rangeSet.add(Range.closedOpen(11, 15)); // disconnected range; {[1, 10], [11, 15)}
    rangeSet.add(Range.closedOpen(15, 20)); // connected range; {[1, 10], [11, 20)}
    rangeSet.add(Range.openClosed(0, 0)); // empty range; {[1, 10], [11, 20)}
    rangeSet.remove(Range.open(5, 10)); // splits [1, 10]; {[1, 5], [10, 10], [11, 20)}
    }```
    
    Note that the behavior of Range.isEmpty() and Range.isConnected(Range) may not
    be as expected on discrete ranges. See the Javadoc of those methods for details.
    
    For a Set whose contents are specified by a Range, see ContiguousSet.
    
    See the Guava User Guide article on <a href=
    "https://github.com/google/guava/wiki/NewCollectionTypesExplained#rangeset">RangeSets</a>.

    Author(s)
    - Louis Wasserman

    Since
    - 14.0
    """

    def contains(self, value: "C") -> bool:
        """
        Determines whether any of this range set's member ranges contains `value`.
        """
        ...


    def rangeContaining(self, value: "C") -> "Range"["C"]:
        """
        Returns the unique range from this range set that Range.contains contains `value`, or `null` if this range set does not contain `value`.
        """
        ...


    def intersects(self, otherRange: "Range"["C"]) -> bool:
        """
        Returns `True` if there exists a non-empty range enclosed by both a member range in this
        range set and the specified range. This is equivalent to calling `subRangeSet(otherRange)` and testing whether the resulting range set is non-empty.

        Since
        - 20.0
        """
        ...


    def encloses(self, otherRange: "Range"["C"]) -> bool:
        """
        Returns `True` if there exists a member range in this range set which Range.encloses encloses the specified range.
        """
        ...


    def enclosesAll(self, other: "RangeSet"["C"]) -> bool:
        """
        Returns `True` if for each member range in `other` there exists a member range in
        this range set which Range.encloses encloses it. It follows that `this.contains(value)` whenever `other.contains(value)`. Returns `True` if `other` is empty.
        
        This is equivalent to checking if this range set .encloses each of the ranges in
        `other`.
        """
        ...


    def enclosesAll(self, other: Iterable["Range"["C"]]) -> bool:
        """
        Returns `True` if for each range in `other` there exists a member range in this
        range set which Range.encloses encloses it. Returns `True` if `other`
        is empty.
        
        This is equivalent to checking if this range set .encloses each range in `other`.

        Since
        - 21.0
        """
        ...


    def isEmpty(self) -> bool:
        """
        Returns `True` if this range set contains no ranges.
        """
        ...


    def span(self) -> "Range"["C"]:
        """
        Returns the minimal range which Range.encloses(Range) encloses all ranges in this
        range set.

        Raises
        - NoSuchElementException: if this range set is .isEmpty() empty
        """
        ...


    def asRanges(self) -> set["Range"["C"]]:
        """
        Returns a view of the Range.isConnected disconnected ranges that make up this
        range set. The returned set may be empty. The iterators returned by its Iterable.iterator method return the ranges in increasing order of lower bound (equivalently,
        of upper bound).
        """
        ...


    def asDescendingSetOfRanges(self) -> set["Range"["C"]]:
        """
        Returns a descending view of the Range.isConnected disconnected ranges that make
        up this range set. The returned set may be empty. The iterators returned by its Iterable.iterator method return the ranges in decreasing order of lower bound (equivalently,
        of upper bound).

        Since
        - 19.0
        """
        ...


    def complement(self) -> "RangeSet"["C"]:
        """
        Returns a view of the complement of this `RangeSet`.
        
        The returned view supports the .add operation if this `RangeSet` supports
        .remove, and vice versa.
        """
        ...


    def subRangeSet(self, view: "Range"["C"]) -> "RangeSet"["C"]:
        """
        Returns a view of the intersection of this `RangeSet` with the specified range.
        
        The returned view supports all optional operations supported by this `RangeSet`, with
        the caveat that an IllegalArgumentException is thrown on an attempt to .add(Range) add any range not Range.encloses(Range) enclosed by `view`.
        """
        ...


    def add(self, range: "Range"["C"]) -> None:
        """
        Adds the specified range to this `RangeSet` (optional operation). That is, for equal
        range sets a and b, the result of `a.add(range)` is that `a` will be the minimal
        range set for which both `a.enclosesAll(b)` and `a.encloses(range)`.
        
        Note that `range` will be Range.span(Range) coalesced with any ranges in
        the range set that are Range.isConnected(Range) connected with it. Moreover, if
        `range` is empty, this is a no-op.

        Raises
        - UnsupportedOperationException: if this range set does not support the `add`
            operation
        """
        ...


    def remove(self, range: "Range"["C"]) -> None:
        """
        Removes the specified range from this `RangeSet` (optional operation). After this
        operation, if `range.contains(c)`, `this.contains(c)` will return `False`.
        
        If `range` is empty, this is a no-op.

        Raises
        - UnsupportedOperationException: if this range set does not support the `remove`
            operation
        """
        ...


    def clear(self) -> None:
        """
        Removes all ranges from this `RangeSet` (optional operation). After this operation,
        `this.contains(c)` will return False for all `c`.
        
        This is equivalent to `remove(Range.all())`.

        Raises
        - UnsupportedOperationException: if this range set does not support the `clear`
            operation
        """
        ...


    def addAll(self, other: "RangeSet"["C"]) -> None:
        """
        Adds all of the ranges from the specified range set to this range set (optional operation).
        After this operation, this range set is the minimal range set that .enclosesAll(RangeSet) encloses both the original range set and `other`.
        
        This is equivalent to calling .add on each of the ranges in `other` in turn.

        Raises
        - UnsupportedOperationException: if this range set does not support the `addAll`
            operation
        """
        ...


    def addAll(self, ranges: Iterable["Range"["C"]]) -> None:
        """
        Adds all of the specified ranges to this range set (optional operation). After this operation,
        this range set is the minimal range set that .enclosesAll(RangeSet) encloses both
        the original range set and each range in `other`.
        
        This is equivalent to calling .add on each of the ranges in `other` in turn.

        Raises
        - UnsupportedOperationException: if this range set does not support the `addAll`
            operation

        Since
        - 21.0
        """
        ...


    def removeAll(self, other: "RangeSet"["C"]) -> None:
        """
        Removes all of the ranges from the specified range set from this range set (optional
        operation). After this operation, if `other.contains(c)`, `this.contains(c)` will
        return `False`.
        
        This is equivalent to calling .remove on each of the ranges in `other` in
        turn.

        Raises
        - UnsupportedOperationException: if this range set does not support the `removeAll`
            operation
        """
        ...


    def removeAll(self, ranges: Iterable["Range"["C"]]) -> None:
        """
        Removes all of the specified ranges from this range set (optional operation).
        
        This is equivalent to calling .remove on each of the ranges in `other` in
        turn.

        Raises
        - UnsupportedOperationException: if this range set does not support the `removeAll`
            operation

        Since
        - 21.0
        """
        ...


    def equals(self, obj: "Object") -> bool:
        """
        Returns `True` if `obj` is another `RangeSet` that contains the same ranges
        according to Range.equals(Object).
        """
        ...


    def hashCode(self) -> int:
        """
        Returns `asRanges().hashCode()`.
        """
        ...


    def toString(self) -> str:
        """
        Returns a readable string representation of this range set. For example, if this `RangeSet` consisted of `Range.closed(1, 3)` and `Range.greaterThan(4)`, this might
        return `" [1..3](4..+∞)`"}.
        """
        ...



"""
Python module generated from Java source file org.bukkit.util.EntityTransformer

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.entity import Entity
from org.bukkit.generator import LimitedRegion
from org.bukkit.util import *
from typing import Any, Callable, Iterable, Tuple


class EntityTransformer:
    """
    A EntityTransformer is used to modify entities that are spawned by structure.
    """

    def transform(self, region: "LimitedRegion", x: int, y: int, z: int, entity: "Entity", allowedToSpawn: bool) -> bool:
        """
        Transforms a entity in a structure.

        Arguments
        - region: the accessible region
        - x: the x position of the entity
        - y: the y position of the entity
        - z: the z position of the entity
        - entity: the entity
        - allowedToSpawn: if the entity is allowed to spawn

        Returns
        - `True` if the entity should be spawned otherwise
        `False`
        """
        ...



"""
Python module generated from Java source file org.bukkit.entity.Mob

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit import Sound
from org.bukkit.entity import *
from org.bukkit.loot import Lootable
from typing import Any, Callable, Iterable, Tuple


class Mob(LivingEntity, Lootable):
    """
    Represents a Mob. Mobs are living entities with simple AI.
    """

    def setTarget(self, target: "LivingEntity") -> None:
        """
        Instructs this Mob to set the specified LivingEntity as its target.
        
        Hostile creatures may attack their target, and friendly creatures may
        follow their target.

        Arguments
        - target: New LivingEntity to target, or null to clear the target
        """
        ...


    def getTarget(self) -> "LivingEntity":
        """
        Gets the current target of this Mob

        Returns
        - Current target of this creature, or null if none exists
        """
        ...


    def setAware(self, aware: bool) -> None:
        """
        Sets whether this mob is aware of its surroundings.
        
        Unaware mobs will still move if pushed, attacked, etc. but will not move
        or perform any actions on their own. Unaware mobs may also have other
        unspecified behaviours disabled, such as drowning.

        Arguments
        - aware: whether the mob is aware
        """
        ...


    def isAware(self) -> bool:
        """
        Gets whether this mob is aware of its surroundings.
        
        Unaware mobs will still move if pushed, attacked, etc. but will not move
        or perform any actions on their own. Unaware mobs may also have other
        unspecified behaviours disabled, such as drowning.

        Returns
        - whether the mob is aware
        """
        ...


    def getAmbientSound(self) -> "Sound":
        """
        Get the Sound this mob makes while ambiently existing. This sound
        may change depending on the current state of the entity, and may also
        return null under specific conditions. This sound is not constant.
        For instance, villagers will make different passive noises depending
        on whether or not they are actively trading with a player, or make no
        ambient noise while sleeping.

        Returns
        - the ambient sound, or null if this entity is ambiently quiet
        """
        ...



"""
Python module generated from Java source file com.google.gson.annotations.JsonAdapter

Java source file obtained from artifact gson version 2.11.0

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.gson import Gson
from com.google.gson import GsonBuilder
from com.google.gson import InstanceCreator
from com.google.gson import JsonDeserializer
from com.google.gson import JsonSerializer
from com.google.gson import TypeAdapter
from com.google.gson import TypeAdapterFactory
from com.google.gson.annotations import *
from typing import Any, Callable, Iterable, Tuple


class JsonAdapter:

    def value(self) -> type[Any]:
        """
        Either a TypeAdapter or TypeAdapterFactory, or one or both of JsonDeserializer or JsonSerializer.
        """
        ...


    def nullSafe(self) -> bool:
        """
        Whether the adapter referenced by .value() should be made TypeAdapter.nullSafe() null-safe.
        
        If `True` (the default), it will be made null-safe and Gson will handle `null`
        Java objects on serialization and JSON `null` on deserialization without calling the
        adapter. If `False`, the adapter will have to handle the `null` values.
        """
        return True



"""
Python module generated from Java source file org.bukkit.inventory.meta.tags.ItemTagAdapterContext

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.inventory.meta.tags import *
from org.bukkit.persistence import PersistentDataAdapterContext
from org.bukkit.persistence import PersistentDataHolder
from typing import Any, Callable, Iterable, Tuple


class ItemTagAdapterContext:
    """
    This interface represents the context in which the ItemTagType can
    serialize and deserialize the passed values.

    Deprecated
    - this API part has been replaced by PersistentDataHolder.
    Please use PersistentDataAdapterContext instead of this.
    """

    def newTagContainer(self) -> "CustomItemTagContainer":
        """
        Creates a new and empty tag container instance.

        Returns
        - the fresh container instance
        """
        ...



"""
Python module generated from Java source file org.bukkit.plugin.ServicesManager

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.plugin import *
from typing import Any, Callable, Iterable, Tuple


class ServicesManager:
    """
    Manages services and service providers. Services are an interface
    specifying a list of methods that a provider must implement. Providers are
    implementations of these services. A provider can be queried from the
    services manager in order to use a service (if one is available). If
    multiple plugins register a service, then the service with the highest
    priority takes precedence.
    """

    def register(self, service: type["T"], provider: "T", plugin: "Plugin", priority: "ServicePriority") -> None:
        """
        Register a provider of a service.
        
        Type `<T>`: Provider

        Arguments
        - service: service class
        - provider: provider to register
        - plugin: plugin with the provider
        - priority: priority of the provider
        """
        ...


    def unregisterAll(self, plugin: "Plugin") -> None:
        """
        Unregister all the providers registered by a particular plugin.

        Arguments
        - plugin: The plugin
        """
        ...


    def unregister(self, service: type[Any], provider: "Object") -> None:
        """
        Unregister a particular provider for a particular service.

        Arguments
        - service: The service interface
        - provider: The service provider implementation
        """
        ...


    def unregister(self, provider: "Object") -> None:
        """
        Unregister a particular provider.

        Arguments
        - provider: The service provider implementation
        """
        ...


    def load(self, service: type["T"]) -> "T":
        """
        Queries for a provider. This may return null if no provider has been
        registered for a service. The highest priority provider is returned.
        
        Type `<T>`: The service interface

        Arguments
        - service: The service interface

        Returns
        - provider or null
        """
        ...


    def getRegistration(self, service: type["T"]) -> "RegisteredServiceProvider"["T"]:
        """
        Queries for a provider registration. This may return null if no provider
        has been registered for a service.
        
        Type `<T>`: The service interface

        Arguments
        - service: The service interface

        Returns
        - provider registration or null
        """
        ...


    def getRegistrations(self, plugin: "Plugin") -> list["RegisteredServiceProvider"[Any]]:
        """
        Get registrations of providers for a plugin.

        Arguments
        - plugin: The plugin

        Returns
        - provider registrations
        """
        ...


    def getRegistrations(self, service: type["T"]) -> Iterable["RegisteredServiceProvider"["T"]]:
        """
        Get registrations of providers for a service. The returned list is
        unmodifiable.
        
        Type `<T>`: The service interface

        Arguments
        - service: The service interface

        Returns
        - list of registrations
        """
        ...


    def getKnownServices(self) -> Iterable[type[Any]]:
        """
        Get a list of known services. A service is known if it has registered
        providers for it.

        Returns
        - list of known services
        """
        ...


    def isProvidedFor(self, service: type["T"]) -> bool:
        """
        Returns whether a provider has been registered for a service. Do not
        check this first only to call `load(service)` later, as that
        would be a non-thread safe situation.
        
        Type `<T>`: service

        Arguments
        - service: service to check

        Returns
        - whether there has been a registered provider
        """
        ...



"""
Python module generated from Java source file org.bukkit.plugin.java.PluginClassLoader

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.base import Preconditions
from com.google.common.io import ByteStreams
from java.io import File
from java.io import IOException
from java.io import InputStream
from java.lang.reflect import Constructor
from java.lang.reflect import InvocationTargetException
from java.net import MalformedURLException
from java.net import URL
from java.net import URLClassLoader
from java.security import CodeSigner
from java.security import CodeSource
from java.util import Collections
from java.util import Enumeration
from java.util.concurrent import ConcurrentHashMap
from java.util.jar import JarEntry
from java.util.jar import JarFile
from java.util.jar import Manifest
from org.bukkit.plugin import InvalidPluginException
from org.bukkit.plugin import PluginDescriptionFile
from org.bukkit.plugin import SimplePluginManager
from org.bukkit.plugin.java import *
from typing import Any, Callable, Iterable, Tuple


class PluginClassLoader(URLClassLoader):
    """
    A ClassLoader for plugins, to allow shared classes across multiple plugins
    """

    def getResource(self, name: str) -> "URL":
        ...


    def getResources(self, name: str) -> "Enumeration"["URL"]:
        ...


    def close(self) -> None:
        ...



"""
Python module generated from Java source file org.bukkit.entity.Minecart

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit import GameRule
from org.bukkit.block.data import BlockData
from org.bukkit.entity import *
from org.bukkit.material import MaterialData
from org.bukkit.util import Vector
from typing import Any, Callable, Iterable, Tuple


class Minecart(Vehicle):
    """
    Represents a minecart entity.
    """

    def setDamage(self, damage: float) -> None:
        """
        Sets a minecart's damage.

        Arguments
        - damage: over 40 to "kill" a minecart
        """
        ...


    def getDamage(self) -> float:
        """
        Gets a minecart's damage.

        Returns
        - The damage
        """
        ...


    def getMaxSpeed(self) -> float:
        """
        Gets the maximum speed of a minecart. The speed is unrelated to the
        velocity.

        Returns
        - The max speed
        """
        ...


    def setMaxSpeed(self, speed: float) -> None:
        """
        Sets the maximum speed of a minecart. Must be nonnegative. Default is
        0.4D or GameRule.MINECART_MAX_SPEED.

        Arguments
        - speed: The max speed
        """
        ...


    def isSlowWhenEmpty(self) -> bool:
        """
        Returns whether this minecart will slow down faster without a passenger
        occupying it

        Returns
        - Whether it decelerates faster
        """
        ...


    def setSlowWhenEmpty(self, slow: bool) -> None:
        """
        Sets whether this minecart will slow down faster without a passenger
        occupying it

        Arguments
        - slow: Whether it will decelerate faster
        """
        ...


    def getFlyingVelocityMod(self) -> "Vector":
        """
        Gets the flying velocity modifier. Used for minecarts that are in
        mid-air. A flying minecart's velocity is multiplied by this factor each
        tick.

        Returns
        - The vector factor
        """
        ...


    def setFlyingVelocityMod(self, flying: "Vector") -> None:
        """
        Sets the flying velocity modifier. Used for minecarts that are in
        mid-air. A flying minecart's velocity is multiplied by this factor each
        tick.

        Arguments
        - flying: velocity modifier vector
        """
        ...


    def getDerailedVelocityMod(self) -> "Vector":
        """
        Gets the derailed velocity modifier. Used for minecarts that are on the
        ground, but not on rails.
        
        A derailed minecart's velocity is multiplied by this factor each tick.

        Returns
        - derailed visible speed
        """
        ...


    def setDerailedVelocityMod(self, derailed: "Vector") -> None:
        """
        Sets the derailed velocity modifier. Used for minecarts that are on the
        ground, but not on rails. A derailed minecart's velocity is multiplied
        by this factor each tick.

        Arguments
        - derailed: visible speed
        """
        ...


    def setDisplayBlock(self, material: "MaterialData") -> None:
        """
        Sets the display block for this minecart.
        Passing a null value will set the minecart to have no display block.

        Arguments
        - material: the material to set as display block.
        """
        ...


    def getDisplayBlock(self) -> "MaterialData":
        """
        Gets the display block for this minecart.
        This function will return the type AIR if none is set.

        Returns
        - the block displayed by this minecart.
        """
        ...


    def setDisplayBlockData(self, blockData: "BlockData") -> None:
        """
        Sets the display block for this minecart.
        Passing a null value will set the minecart to have no display block.

        Arguments
        - blockData: the material to set as display block.
        """
        ...


    def getDisplayBlockData(self) -> "BlockData":
        """
        Gets the display block for this minecart.
        This function will return the type AIR if none is set.

        Returns
        - the block displayed by this minecart.
        """
        ...


    def setDisplayBlockOffset(self, offset: int) -> None:
        """
        Sets the offset of the display block.

        Arguments
        - offset: the block offset to set for this minecart.
        """
        ...


    def getDisplayBlockOffset(self) -> int:
        """
        Gets the offset of the display block.

        Returns
        - the current block offset for this minecart.
        """
        ...



"""
Python module generated from Java source file org.bukkit.plugin.messaging.PluginMessageRecipient

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.plugin import Plugin
from org.bukkit.plugin.messaging import *
from typing import Any, Callable, Iterable, Tuple


class PluginMessageRecipient:
    """
    Represents a possible recipient for a Plugin Message.
    """

    def sendPluginMessage(self, source: "Plugin", channel: str, message: list[int]) -> None:
        """
        Sends this recipient a Plugin Message on the specified outgoing
        channel.
        
        The message may not be larger than Messenger.MAX_MESSAGE_SIZE
        bytes, and the plugin must be registered to send messages on the
        specified channel.

        Arguments
        - source: The plugin that sent this message.
        - channel: The channel to send this message on.
        - message: The raw message to send.

        Raises
        - IllegalArgumentException: Thrown if the source plugin is
            disabled.
        - IllegalArgumentException: Thrown if source, channel or message
            is null.
        - MessageTooLargeException: Thrown if the message is too big.
        - ChannelNotRegisteredException: Thrown if the channel is not
            registered for this plugin.
        """
        ...


    def getListeningPluginChannels(self) -> set[str]:
        """
        Gets a set containing all the Plugin Channels that this client is
        listening on.

        Returns
        - Set containing all the channels that this client may accept.
        """
        ...



"""
Python module generated from Java source file com.google.common.collect.JdkBackedImmutableBiMap

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.annotations import GwtIncompatible
from com.google.common.annotations import J2ktIncompatible
from com.google.common.collect import *
from com.google.errorprone.annotations.concurrent import LazyInit
from com.google.j2objc.annotations import RetainedWith
from com.google.j2objc.annotations import WeakOuter
from javax.annotation import CheckForNull
from org.checkerframework.checker.nullness.qual import Nullable
from typing import Any, Callable, Iterable, Tuple


class JdkBackedImmutableBiMap(ImmutableBiMap):
    """
    Implementation of ImmutableBiMap backed by a pair of JDK HashMaps, which have smartness
    protecting against hash flooding.
    """

    def size(self) -> int:
        ...


    def inverse(self) -> "ImmutableBiMap"["V", "K"]:
        ...


    def get(self, key: "Object") -> "V":
        ...



"""
Python module generated from Java source file org.bukkit.GameEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.collect import Lists
from java.util import Collections
from org.bukkit import *
from org.bukkit.registry import RegistryAware
from typing import Any, Callable, Iterable, Tuple


class GameEvent(Keyed, RegistryAware):
    """
    Represents a generic Mojang game event.
    """

    BLOCK_ACTIVATE = getEvent("block_activate")
    BLOCK_ATTACH = getEvent("block_attach")
    BLOCK_CHANGE = getEvent("block_change")
    BLOCK_CLOSE = getEvent("block_close")
    BLOCK_DEACTIVATE = getEvent("block_deactivate")
    BLOCK_DESTROY = getEvent("block_destroy")
    BLOCK_DETACH = getEvent("block_detach")
    BLOCK_OPEN = getEvent("block_open")
    BLOCK_PLACE = getEvent("block_place")
    BLOCK_PRESS = getEvent("block_activate")
    BLOCK_SWITCH = getEvent("block_activate")
    BLOCK_UNPRESS = getEvent("block_deactivate")
    BLOCK_UNSWITCH = getEvent("block_deactivate")
    CONTAINER_CLOSE = getEvent("container_close")
    CONTAINER_OPEN = getEvent("container_open")
    DISPENSE_FAIL = getEvent("block_activate")
    DRINK = getEvent("drink")
    DRINKING_FINISH = getEvent("drink")
    EAT = getEvent("eat")
    ELYTRA_FREE_FALL = getEvent("elytra_glide")
    ELYTRA_GLIDE = getEvent("elytra_glide")
    ENTITY_DAMAGE = getEvent("entity_damage")
    ENTITY_DAMAGED = getEvent("entity_damage")
    ENTITY_DIE = getEvent("entity_die")
    ENTITY_DISMOUNT = getEvent("entity_dismount")
    ENTITY_DYING = getEvent("entity_die")
    ENTITY_INTERACT = getEvent("entity_interact")
    ENTITY_MOUNT = getEvent("entity_mount")
    ENTITY_KILLED = getEvent("entity_die")
    ENTITY_PLACE = getEvent("entity_place")
    ENTITY_ACTION = getEvent("entity_action")
    ENTITY_ROAR = getEvent("entity_action")
    ENTITY_SHAKE = getEvent("entity_action")
    EQUIP = getEvent("equip")
    EXPLODE = getEvent("explode")
    FLAP = getEvent("flap")
    FLUID_PICKUP = getEvent("fluid_pickup")
    FLUID_PLACE = getEvent("fluid_place")
    HIT_GROUND = getEvent("hit_ground")
    INSTRUMENT_PLAY = getEvent("instrument_play")
    ITEM_INTERACT_FINISH = getEvent("item_interact_finish")
    ITEM_INTERACT_START = getEvent("item_interact_start")
    JUKEBOX_PLAY = getEvent("jukebox_play")
    JUKEBOX_STOP_PLAY = getEvent("jukebox_stop_play")
    LIGHTNING_STRIKE = getEvent("lightning_strike")
    MOB_INTERACT = getEvent("entity_interact")
    NOTE_BLOCK_PLAY = getEvent("note_block_play")
    PISTON_CONTRACT = getEvent("block_deactivate")
    PISTON_EXTEND = getEvent("block_activate")
    PRIME_FUSE = getEvent("prime_fuse")
    PROJECTILE_LAND = getEvent("projectile_land")
    PROJECTILE_SHOOT = getEvent("projectile_shoot")
    RAVAGER_ROAR = getEvent("entity_action")
    RING_BELL = getEvent("block_change")
    SCULK_SENSOR_TENDRILS_CLICKING = getEvent("sculk_sensor_tendrils_clicking")
    SHEAR = getEvent("shear")
    SHRIEK = getEvent("shriek")
    SHULKER_CLOSE = getEvent("container_close")
    SHULKER_OPEN = getEvent("container_open")
    SPLASH = getEvent("splash")
    STEP = getEvent("step")
    SWIM = getEvent("swim")
    TELEPORT = getEvent("teleport")
    UNEQUIP = getEvent("unequip")
    WOLF_SHAKING = getEvent("entity_action")
    RESONATE_1 = getEvent("resonate_1")
    RESONATE_2 = getEvent("resonate_2")
    RESONATE_3 = getEvent("resonate_3")
    RESONATE_4 = getEvent("resonate_4")
    RESONATE_5 = getEvent("resonate_5")
    RESONATE_6 = getEvent("resonate_6")
    RESONATE_7 = getEvent("resonate_7")
    RESONATE_8 = getEvent("resonate_8")
    RESONATE_9 = getEvent("resonate_9")
    RESONATE_10 = getEvent("resonate_10")
    RESONATE_11 = getEvent("resonate_11")
    RESONATE_12 = getEvent("resonate_12")
    RESONATE_13 = getEvent("resonate_13")
    RESONATE_14 = getEvent("resonate_14")
    RESONATE_15 = getEvent("resonate_15")


    def getKey(self) -> "NamespacedKey":
        """
        See
        - .isRegistered()

        Deprecated
        - A key might not always be present, use .getKeyOrThrow() instead.
        """
        ...


    @staticmethod
    def getByKey(namespacedKey: "NamespacedKey") -> "GameEvent":
        """
        Returns a GameEvent by a NamespacedKey.

        Arguments
        - namespacedKey: the key

        Returns
        - the event or null

        Deprecated
        - Use Registry.get(NamespacedKey) instead.
        """
        ...


    @staticmethod
    def values() -> Iterable["GameEvent"]:
        """
        Returns the set of all GameEvents.

        Returns
        - the memoryKeys

        Deprecated
        - use Registry.iterator().
        """
        ...



"""
Python module generated from Java source file com.google.common.net.PercentEscaper

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.escape import UnicodeEscaper
from com.google.common.net import *
from javax.annotation import CheckForNull
from typing import Any, Callable, Iterable, Tuple


class PercentEscaper(UnicodeEscaper):
    """
    A `UnicodeEscaper` that escapes some set of Java characters using a UTF-8 based percent
    encoding scheme. The set of safe characters (those which remain unescaped) can be specified on
    construction.
    
    This class is primarily used for creating URI escapers in UrlEscapers but can be used
    directly if required. While URI escapers impose specific semantics on which characters are
    considered 'safe', this class has a minimal set of restrictions.
    
    When escaping a String, the following rules apply:
    
    
      - All specified safe characters remain unchanged.
      - If `plusForSpace` was specified, the space character " " is converted into a plus
          sign `"+"`.
      - All other characters are converted into one or more bytes using UTF-8 encoding and each
          byte is then represented by the 3-character string "%XX", where "XX" is the two-digit,
          uppercase, hexadecimal representation of the byte value.
    
    
    For performance reasons the only currently supported character encoding of this class is
    UTF-8.
    
    **Note:** This escaper produces <a
    href="https://url.spec.whatwg.org/#percent-encode">uppercase</a> hexadecimal sequences.

    Author(s)
    - David Beaumont

    Since
    - 15.0
    """

    def __init__(self, safeChars: str, plusForSpace: bool):
        """
        Constructs a percent escaper with the specified safe characters and optional handling of the
        space character.
        
        Not that it is allowed, but not necessarily desirable to specify `%` as a safe
        character. This has the effect of creating an escaper which has no well-defined inverse but it
        can be useful when escaping additional characters.

        Arguments
        - safeChars: a non-null string specifying additional safe characters for this escaper (the
            ranges 0..9, a..z and A..Z are always safe and should not be specified here)
        - plusForSpace: True if ASCII space should be escaped to `+` rather than `%20`

        Raises
        - IllegalArgumentException: if any of the parameters were invalid
        """
        ...


    def escape(self, s: str) -> str:
        ...



"""
Python module generated from Java source file com.google.gson.internal.TroubleshootingGuide

Java source file obtained from artifact gson version 2.11.0

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.gson.internal import *
from typing import Any, Callable, Iterable, Tuple


class TroubleshootingGuide:

    @staticmethod
    def createUrl(id: str) -> str:
        """
        Creates a URL referring to the specified troubleshooting section.
        """
        ...



"""
Python module generated from Java source file java.nio.charset.CodingErrorAction

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.nio.charset import *
from typing import Any, Callable, Iterable, Tuple


class CodingErrorAction:

    IGNORE = CodingErrorAction("IGNORE")
    """
    Action indicating that a coding error is to be handled by dropping the
    erroneous input and resuming the coding operation.
    """
    REPLACE = CodingErrorAction("REPLACE")
    """
    Action indicating that a coding error is to be handled by dropping the
    erroneous input, appending the coder's replacement value to the output
    buffer, and resuming the coding operation.
    """
    REPORT = CodingErrorAction("REPORT")
    """
    Action indicating that a coding error is to be reported, either by
    returning a CoderResult object or by throwing a CharacterCodingException, whichever is appropriate for the method
    implementing the coding process.
    """


    def toString(self) -> str:
        """
        Returns a string describing this action.

        Returns
        - A descriptive string
        """
        ...



from .GenericCommandHelpTopic import GenericCommandHelpTopic
from .HelpMap import HelpMap
from .HelpTopic import HelpTopic
from .IndexHelpTopic import IndexHelpTopic
from .HelpTopicComparator import HelpTopicComparator
from .HelpTopicFactory import HelpTopicFactory


"""
Python module generated from Java source file java.nio.file.FileAlreadyExistsException

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.nio.file import *
from typing import Any, Callable, Iterable, Tuple


class FileAlreadyExistsException(FileSystemException):

    def __init__(self, file: str):
        """
        Constructs an instance of this class.

        Arguments
        - file: a string identifying the file or `null` if not known
        """
        ...


    def __init__(self, file: str, other: str, reason: str):
        """
        Constructs an instance of this class.

        Arguments
        - file: a string identifying the file or `null` if not known
        - other: a string identifying the other file or `null` if not known
        - reason: a reason message with additional information or `null`
        """
        ...



"""
Python module generated from Java source file org.bukkit.event.entity.EntityDamageEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.base import Function
from com.google.common.base import Functions
from com.google.common.base import Preconditions
from com.google.common.collect import ImmutableMap
from enum import Enum
from java.util import EnumMap
from java.util import Objects
from org.bukkit import Material
from org.bukkit import WorldBorder
from org.bukkit.damage import DamageSource
from org.bukkit.damage import DamageType
from org.bukkit.entity import Entity
from org.bukkit.entity import Player
from org.bukkit.event import Cancellable
from org.bukkit.event import HandlerList
from org.bukkit.event.entity import *
from typing import Any, Callable, Iterable, Tuple


class EntityDamageEvent(EntityEvent, Cancellable):
    """
    Stores data for damage events
    """

    def __init__(self, damagee: "Entity", cause: "DamageCause", damage: float):
        ...


    def __init__(self, damagee: "Entity", cause: "DamageCause", damageSource: "DamageSource", damage: float):
        ...


    def __init__(self, damagee: "Entity", cause: "DamageCause", modifiers: dict["DamageModifier", "Double"], modifierFunctions: dict["DamageModifier", "Function"["Double", "Double"]]):
        ...


    def __init__(self, damagee: "Entity", cause: "DamageCause", damageSource: "DamageSource", modifiers: dict["DamageModifier", "Double"], modifierFunctions: dict["DamageModifier", "Function"["Double", "Double"]]):
        ...


    def isCancelled(self) -> bool:
        ...


    def setCancelled(self, cancel: bool) -> None:
        ...


    def getOriginalDamage(self, type: "DamageModifier") -> float:
        """
        Gets the original damage for the specified modifier, as defined at this
        event's construction.

        Arguments
        - type: the modifier

        Returns
        - the original damage

        Raises
        - IllegalArgumentException: if type is null
        """
        ...


    def setDamage(self, type: "DamageModifier", damage: float) -> None:
        """
        Sets the damage for the specified modifier.

        Arguments
        - type: the damage modifier
        - damage: the scalar value of the damage's modifier

        Raises
        - IllegalArgumentException: if type is null
        - UnsupportedOperationException: if the caller does not support
            the particular DamageModifier, or to rephrase, when .isApplicable(DamageModifier) returns False

        See
        - .getFinalDamage()
        """
        ...


    def getDamage(self, type: "DamageModifier") -> float:
        """
        Gets the damage change for some modifier

        Arguments
        - type: the damage modifier

        Returns
        - The raw amount of damage caused by the event

        Raises
        - IllegalArgumentException: if type is null

        See
        - DamageModifier.BASE
        """
        ...


    def isApplicable(self, type: "DamageModifier") -> bool:
        """
        This checks to see if a particular modifier is valid for this event's
        caller, such that, .setDamage(DamageModifier, double) will not
        throw an UnsupportedOperationException.
        
        DamageModifier.BASE is always applicable.

        Arguments
        - type: the modifier

        Returns
        - True if the modifier is supported by the caller, False otherwise

        Raises
        - IllegalArgumentException: if type is null
        """
        ...


    def getDamage(self) -> float:
        """
        Gets the raw amount of damage caused by the event

        Returns
        - The raw amount of damage caused by the event

        See
        - DamageModifier.BASE
        """
        ...


    def getFinalDamage(self) -> float:
        """
        Gets the amount of damage caused by the event after all damage
        reduction is applied.

        Returns
        - the amount of damage caused by the event
        """
        ...


    def setDamage(self, damage: float) -> None:
        """
        Sets the raw amount of damage caused by the event.
        
        For compatibility this also recalculates the modifiers and scales
        them by the difference between the modifier for the previous damage
        value and the new one.

        Arguments
        - damage: The raw amount of damage caused by the event
        """
        ...


    def getCause(self) -> "DamageCause":
        """
        Gets the cause of the damage.
        
        While a DamageCause may indicate a specific Bukkit-assigned cause of damage,
        .getDamageSource() may expose additional types of damage such as custom
        damage types provided by data packs, as well as any direct or indirect entities,
        locations, or other contributing factors to the damage being inflicted. The
        alternative is generally preferred, but DamageCauses provided to this event
        should largely encompass most common use cases for developers if a simple cause
        is required.

        Returns
        - a DamageCause value detailing the cause of the damage.
        """
        ...


    def getDamageSource(self) -> "DamageSource":
        """
        Get the source of damage.

        Returns
        - a DamageSource detailing the source of the damage.
        """
        ...


    def getHandlers(self) -> "HandlerList":
        ...


    @staticmethod
    def getHandlerList() -> "HandlerList":
        ...


    class DamageModifier(Enum):
        """
        An enum to specify the types of modifier

        Deprecated
        - This API is responsible for a large number of implementation
        problems and is in general unsustainable to maintain. It is likely to be
        removed very soon in a subsequent release. Please see
        <a href="https://www.spigotmc.org/threads/194446/">this thread</a> for more information.
        """

        BASE = 0
        """
        This represents the amount of damage being done, also known as the
        raw EntityDamageEvent.getDamage().
        """
        FREEZING = 1
        """
        This represents the damage increased by freezing status.
        """
        HARD_HAT = 2
        """
        This represents the damage reduced by a wearing a helmet when hit
        by a falling block.
        """
        BLOCKING = 3
        """
        This represents  the damage reduction caused by blocking, only present for
        Player Players.
        """
        ARMOR = 4
        """
        This represents the damage reduction caused by wearing armor.
        """
        RESISTANCE = 5
        """
        This represents the damage reduction caused by the Resistance potion effect.
        """
        MAGIC = 6
        """
        This represents the damage reduction caused by the combination of:
        
        - 
            Armor enchantments
        - 
            Witch's potion resistance
        
        """
        ABSORPTION = 7
        """
        This represents the damage reduction caused by the absorption potion
        effect.
        """


    class DamageCause(Enum):
        """
        An enum to specify the cause of the damage
        """

        KILL = 0
        """
        Damage caused by /kill command
        
        Damage: Float.MAX_VALUE
        """
        WORLD_BORDER = 1
        """
        Damage caused by the World Border
        
        Damage: WorldBorder.getDamageAmount()
        """
        CONTACT = 2
        """
        Damage caused when an entity contacts a block such as a Cactus,
        Dripstone (Stalagmite) or Berry Bush.
        
        Damage: variable
        """
        ENTITY_ATTACK = 3
        """
        Damage caused when an entity attacks another entity.
        
        Damage: variable
        """
        ENTITY_SWEEP_ATTACK = 4
        """
        Damage caused when an entity attacks another entity in a sweep attack.
        
        Damage: variable
        """
        PROJECTILE = 5
        """
        Damage caused when attacked by a projectile.
        
        Damage: variable
        """
        SUFFOCATION = 6
        """
        Damage caused by being put in a block
        
        Damage: 1
        """
        FALL = 7
        """
        Damage caused when an entity falls a distance greater than 3 blocks
        
        Damage: fall height - 3.0
        """
        FIRE = 8
        """
        Damage caused by direct exposure to fire
        
        Damage: 1
        """
        FIRE_TICK = 9
        """
        Damage caused due to burns caused by fire
        
        Damage: 1
        """
        MELTING = 10
        """
        Damage caused due to a snowman melting
        
        Damage: 1
        """
        LAVA = 11
        """
        Damage caused by direct exposure to lava
        
        Damage: 4
        """
        DROWNING = 12
        """
        Damage caused by running out of air while in water
        
        Damage: 2
        """
        BLOCK_EXPLOSION = 13
        """
        Damage caused by being in the area when a block explodes.
        
        Damage: variable
        """
        ENTITY_EXPLOSION = 14
        """
        Damage caused by being in the area when an entity, such as a
        Creeper, explodes.
        
        Damage: variable
        """
        VOID = 15
        """
        Damage caused by falling into the void
        
        Damage: 4 for players
        """
        LIGHTNING = 16
        """
        Damage caused by being struck by lightning
        
        Damage: 5
        """
        SUICIDE = 17
        """
        Damage caused by committing suicide.
        
        **Note:** This is currently only used by plugins, default commands
        like /minecraft:kill use .KILL to damage players.
        
        Damage: variable
        """
        STARVATION = 18
        """
        Damage caused by starving due to having an empty hunger bar
        
        Damage: 1
        """
        POISON = 19
        """
        Damage caused due to an ongoing poison effect
        
        Damage: 1
        """
        MAGIC = 20
        """
        Damage caused by being hit by a damage potion or spell
        
        Damage: variable
        """
        WITHER = 21
        """
        Damage caused by Wither potion effect
        """
        FALLING_BLOCK = 22
        """
        Damage caused by being hit by a falling block which deals damage
        
        **Note:** Not every block deals damage
        
        Damage: variable
        """
        THORNS = 23
        """
        Damage caused in retaliation to another attack by the Thorns
        enchantment.
        
        Damage: 1-4 (Thorns)
        """
        DRAGON_BREATH = 24
        """
        Damage caused by a dragon breathing fire.
        
        Damage: variable
        """
        CUSTOM = 25
        """
        Custom damage.
        
        Damage: variable
        """
        FLY_INTO_WALL = 26
        """
        Damage caused when an entity runs into a wall.
        
        Damage: variable
        """
        HOT_FLOOR = 27
        """
        Damage caused when an entity steps on Material.MAGMA_BLOCK.
        
        Damage: 1
        """
        CAMPFIRE = 28
        """
        Damage caused when an entity steps on Material.CAMPFIRE or Material.SOUL_CAMPFIRE.
        
        Damage: 1
        """
        CRAMMING = 29
        """
        Damage caused when an entity is colliding with too many entities due
        to the maxEntityCramming game rule.
        
        Damage: 6
        """
        DRYOUT = 30
        """
        Damage caused when an entity that should be in water is not.
        
        Damage: 1
        """
        FREEZE = 31
        """
        Damage caused from freezing.
        
        Damage: 1 or 5
        """
        SONIC_BOOM = 32
        """
        Damage caused by the Sonic Boom attack from org.bukkit.entity.Warden
        
        Damage: 10
        """



from .Sapling import Sapling
from .CocoaPlant import CocoaPlant
from .Wood import Wood
from .PressureSensor import PressureSensor
from .WoodenStep import WoodenStep
from .Pumpkin import Pumpkin
from .Rails import Rails
from .Comparator import Comparator
from .LongGrass import LongGrass
from .Step import Step
from .Attachable import Attachable
from .DetectorRail import DetectorRail
from .Dye import Dye
from .Sign import Sign
from .RedstoneWire import RedstoneWire
from .Observer import Observer
from .Crops import Crops
from .Hopper import Hopper
from .Command import Command
from .MaterialData import MaterialData
from .Redstone import Redstone
from .Cauldron import Cauldron
from .Colorable import Colorable
from .Skull import Skull
from .SmoothBrick import SmoothBrick
from .Stairs import Stairs
from .Openable import Openable
from .TripwireHook import TripwireHook
from .Sandstone import Sandstone
from .Torch import Torch
from .Lever import Lever
from .MonsterEggs import MonsterEggs
from .Chest import Chest
from .DirectionalContainer import DirectionalContainer
from .Button import Button
from .RedstoneTorch import RedstoneTorch
from .NetherWarts import NetherWarts
from .PistonExtensionMaterial import PistonExtensionMaterial
from .Dispenser import Dispenser
from .SpawnEgg import SpawnEgg
from .Mushroom import Mushroom
from .Banner import Banner
from .Tripwire import Tripwire
from .Furnace import Furnace
from .EnderChest import EnderChest
from .SimpleAttachableMaterialData import SimpleAttachableMaterialData
from .Gate import Gate
from .Tree import Tree
from .Leaves import Leaves
from .PressurePlate import PressurePlate
from .PistonBaseMaterial import PistonBaseMaterial
from .Coal import Coal
from .Bed import Bed
from .Door import Door
from .Directional import Directional
from .PoweredRail import PoweredRail
from .Cake import Cake
from .Diode import Diode
from .FurnaceAndDispenser import FurnaceAndDispenser
from .TexturedMaterial import TexturedMaterial
from .TrapDoor import TrapDoor
from .Wool import Wool
from .FlowerPot import FlowerPot
from .ExtendedRails import ExtendedRails
from .Ladder import Ladder
from .Vine import Vine


"""
Python module generated from Java source file org.bukkit.event.world.AsyncStructureGenerateEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.base import Preconditions
from enum import Enum
from java.util import Collections
from org.bukkit import NamespacedKey
from org.bukkit import World
from org.bukkit.event import HandlerList
from org.bukkit.event.world import *
from org.bukkit.generator.structure import Structure
from org.bukkit.util import BlockTransformer
from org.bukkit.util import BoundingBox
from org.bukkit.util import EntityTransformer
from typing import Any, Callable, Iterable, Tuple


class AsyncStructureGenerateEvent(WorldEvent):
    """
    This event will sometimes fire synchronously, depending on how it was
    triggered.
    
    The constructor provides a boolean to indicate if the event was fired
    synchronously or asynchronously. When asynchronous, this event can be called
    from any thread, sans the main thread, and has limited access to the API.
    
    If a Structure is naturally placed in a chunk of the world, this
    event will be asynchronous. If a player executes the '/place structure'
    command, this event will be synchronous.
    
    Allows to register transformers that can modify the blocks placed and
    entities spawned by the structure.
    
    Care should be taken to check .isAsynchronous() and treat the event
    appropriately.
    """

    def __init__(self, world: "World", async: bool, cause: "Cause", structure: "Structure", boundingBox: "BoundingBox", chunkX: int, chunkZ: int):
        ...


    def getCause(self) -> "Cause":
        """
        Gets the event cause.

        Returns
        - the event cause
        """
        ...


    def getBlockTransformer(self, key: "NamespacedKey") -> "BlockTransformer":
        """
        Gets a block transformer by key.

        Arguments
        - key: the key of the block transformer

        Returns
        - the block transformer or null
        """
        ...


    def setBlockTransformer(self, key: "NamespacedKey", transformer: "BlockTransformer") -> None:
        """
        Sets a block transformer to a key.

        Arguments
        - key: the key
        - transformer: the block transformer
        """
        ...


    def removeBlockTransformer(self, key: "NamespacedKey") -> None:
        """
        Removes a block transformer.

        Arguments
        - key: the key of the block transformer
        """
        ...


    def clearBlockTransformers(self) -> None:
        """
        Removes all block transformers.
        """
        ...


    def getBlockTransformers(self) -> dict["NamespacedKey", "BlockTransformer"]:
        """
        Gets all block transformers in a unmodifiable map.

        Returns
        - the block transformers in a map
        """
        ...


    def getEntityTransformer(self, key: "NamespacedKey") -> "EntityTransformer":
        """
        Gets a entity transformer by key.

        Arguments
        - key: the key of the entity transformer

        Returns
        - the entity transformer or null
        """
        ...


    def setEntityTransformer(self, key: "NamespacedKey", transformer: "EntityTransformer") -> None:
        """
        Sets a entity transformer to a key.

        Arguments
        - key: the key
        - transformer: the entity transformer
        """
        ...


    def removeEntityTransformer(self, key: "NamespacedKey") -> None:
        """
        Removes a entity transformer.

        Arguments
        - key: the key of the entity transformer
        """
        ...


    def clearEntityTransformers(self) -> None:
        """
        Removes all entity transformers.
        """
        ...


    def getEntityTransformers(self) -> dict["NamespacedKey", "EntityTransformer"]:
        """
        Gets all entity transformers in a unmodifiable map.

        Returns
        - the entity transformers in a map
        """
        ...


    def getStructure(self) -> "Structure":
        """
        Get the structure reference that is generated.

        Returns
        - the structure
        """
        ...


    def getBoundingBox(self) -> "BoundingBox":
        """
        Get the bounding box of the structure.

        Returns
        - the bounding box
        """
        ...


    def getChunkX(self) -> int:
        """
        Get the x coordinate of the origin chunk of the structure.

        Returns
        - the chunk x coordinate
        """
        ...


    def getChunkZ(self) -> int:
        """
        Get the z coordinate of the origin chunk of the structure.

        Returns
        - the chunk z coordinate
        """
        ...


    def getHandlers(self) -> "HandlerList":
        ...


    @staticmethod
    def getHandlerList() -> "HandlerList":
        ...


    class Cause(Enum):

        COMMAND = 0
        WORLD_GENERATION = 1
        CUSTOM = 2



"""
Python module generated from Java source file org.bukkit.event.player.PlayerExpChangeEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.entity import Player
from org.bukkit.event import HandlerList
from org.bukkit.event.player import *
from typing import Any, Callable, Iterable, Tuple


class PlayerExpChangeEvent(PlayerEvent):
    """
    Called when a players experience changes naturally
    """

    def __init__(self, player: "Player", expAmount: int):
        ...


    def getAmount(self) -> int:
        """
        Get the amount of experience the player will receive

        Returns
        - The amount of experience
        """
        ...


    def setAmount(self, amount: int) -> None:
        """
        Set the amount of experience the player will receive

        Arguments
        - amount: The amount of experience to set
        """
        ...


    def getHandlers(self) -> "HandlerList":
        ...


    @staticmethod
    def getHandlerList() -> "HandlerList":
        ...



"""
Python module generated from Java source file com.google.common.collect.ArrayListMultimapGwtSerializationDependencies

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.collect import *
from typing import Any, Callable, Iterable, Tuple


class ArrayListMultimapGwtSerializationDependencies(AbstractListMultimap):
    """
    A dummy superclass to support GWT serialization of the element types of an ArrayListMultimap. The GWT supersource for this class contains a field for each type.
    
    For details about this hack, see `GwtSerializationDependencies`, which takes the same
    approach but with a subclass rather than a superclass.
    
    TODO(cpovirk): Consider applying this subclass approach to our other types.
    """





"""
Python module generated from Java source file org.bukkit.event.block.BlockShearEntityEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.block import Block
from org.bukkit.entity import Entity
from org.bukkit.event import Cancellable
from org.bukkit.event import HandlerList
from org.bukkit.event.block import *
from org.bukkit.inventory import ItemStack
from typing import Any, Callable, Iterable, Tuple


class BlockShearEntityEvent(BlockEvent, Cancellable):
    """
    Event fired when a dispenser shears a nearby sheep.
    """

    def __init__(self, dispenser: "Block", sheared: "Entity", tool: "ItemStack"):
        ...


    def getEntity(self) -> "Entity":
        """
        Gets the entity that was sheared.

        Returns
        - the entity that was sheared.
        """
        ...


    def getTool(self) -> "ItemStack":
        """
        Gets the item used to shear this sheep.

        Returns
        - the item used to shear this sheep.
        """
        ...


    def isCancelled(self) -> bool:
        ...


    def setCancelled(self, cancelled: bool) -> None:
        ...


    def getHandlers(self) -> "HandlerList":
        ...


    @staticmethod
    def getHandlerList() -> "HandlerList":
        ...



"""
Python module generated from Java source file org.bukkit.packs.DataPack

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from enum import Enum
from org.bukkit import FeatureFlag
from org.bukkit import Keyed
from org.bukkit.packs import *
from typing import Any, Callable, Iterable, Tuple


class DataPack(Keyed):
    """
    Represents a data pack.

    See
    - <a href="https://minecraft.wiki/w/Data_pack">Minecraft wiki</a>
    """

    def getTitle(self) -> str:
        """
        Gets the title of the data pack.

        Returns
        - the title
        """
        ...


    def getDescription(self) -> str:
        """
        Gets the description of the data pack.

        Returns
        - the description
        """
        ...


    def getPackFormat(self) -> int:
        """
        Gets the pack format.
        
        Pack formats are non-standard and unrelated to the version of Minecraft. For
        a list of known pack versions, see the
        <a href="https://minecraft.wiki/w/Data_pack#Pack_format">Minecraft Wiki</a>.

        Returns
        - the pack version

        See
        - .getMaxSupportedPackFormat()
        """
        ...


    def getMinSupportedPackFormat(self) -> int:
        """
        Gets the minimum supported pack format. If the data pack does not specify a
        minimum supported format, .getPackFormat() is returned.
        
        Pack formats are non-standard and unrelated to the version of Minecraft. For
        a list of known pack versions, see the
        <a href="https://minecraft.wiki/w/Data_pack#Pack_format">Minecraft Wiki</a>.

        Returns
        - the min pack version supported
        """
        ...


    def getMaxSupportedPackFormat(self) -> int:
        """
        Gets the maximum supported pack format. If the data pack does not specify a
        maximum supported format, .getPackFormat() is returned.
        
        Pack formats are non-standard and unrelated to the version of Minecraft. For
        a list of known pack versions, see the
        <a href="https://minecraft.wiki/w/Data_pack#Pack_format">Minecraft Wiki</a>.

        Returns
        - the max pack version supported
        """
        ...


    def isEnabled(self) -> bool:
        """
        Gets if the data pack is enabled on the server.

        Returns
        - True if is enabled
        """
        ...


    def isRequired(self) -> bool:
        """
        Gets if the data pack is required on the server.

        Returns
        - True if is required
        """
        ...


    def getCompatibility(self) -> "Compatibility":
        """
        Gets the compatibility of this data pack with the server.

        Returns
        - an enum
        """
        ...


    def getRequestedFeatures(self) -> set["FeatureFlag"]:
        """
        Gets a set of features requested by this data pack.

        Returns
        - a set of features
        """
        ...


    def getSource(self) -> "Source":
        """
        Gets the source of this data pack.

        Returns
        - the source
        """
        ...


    class Compatibility(Enum):
        """
        Show the compatibility of the data pack with the server.
        """

        NEW = 0
        """
        It's newer than the server pack version.
        """
        OLD = 1
        """
        It's older than the server pack version.
        """
        COMPATIBLE = 2
        """
        Its compatible with the server pack version.
        """


    class Source(Enum):
        """
        Represent the source of a data pack.
        """

        DEFAULT = 0
        BUILT_IN = 1
        FEATURE = 2
        WORLD = 3
        SERVER = 4



"""
Python module generated from Java source file org.bukkit.event.player.PlayerBucketEntityEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit import Material
from org.bukkit.entity import Entity
from org.bukkit.entity import Player
from org.bukkit.event import Cancellable
from org.bukkit.event import HandlerList
from org.bukkit.event.player import *
from org.bukkit.inventory import EquipmentSlot
from org.bukkit.inventory import ItemStack
from typing import Any, Callable, Iterable, Tuple


class PlayerBucketEntityEvent(PlayerEvent, Cancellable):
    """
    This event is called whenever a player captures an entity in a bucket.
    """

    def __init__(self, player: "Player", entity: "Entity", originalBucket: "ItemStack", entityBucket: "ItemStack", hand: "EquipmentSlot"):
        ...


    def getEntity(self) -> "Entity":
        """
        Gets the Entity being put into the bucket.

        Returns
        - The Entity being put into the bucket
        """
        ...


    def getOriginalBucket(self) -> "ItemStack":
        """
        Gets the bucket used to capture the Entity.
        
        This refers to the bucket clicked with, eg Material.WATER_BUCKET.

        Returns
        - The used bucket
        """
        ...


    def getEntityBucket(self) -> "ItemStack":
        """
        Gets the bucket that the Entity will be put into.
        
        This refers to the bucket with the entity, eg
        Material.PUFFERFISH_BUCKET.

        Returns
        - The bucket that the Entity will be put into
        """
        ...


    def getHand(self) -> "EquipmentSlot":
        """
        Get the hand that was used to bucket the entity.

        Returns
        - the hand
        """
        ...


    def isCancelled(self) -> bool:
        ...


    def setCancelled(self, cancel: bool) -> None:
        ...


    def getHandlers(self) -> "HandlerList":
        ...


    @staticmethod
    def getHandlerList() -> "HandlerList":
        ...



"""
Python module generated from Java source file org.yaml.snakeyaml.representer.Represent

Java source file obtained from artifact snakeyaml version 2.2

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.yaml.snakeyaml.nodes import Node
from org.yaml.snakeyaml.representer import *
from typing import Any, Callable, Iterable, Tuple


class Represent:
    """
    Create a Node Graph out of the provided Native Data Structure (Java instance).

    See
    - <a href="http://yaml.org/spec/1.1/.id859109">Chapter 3. Processing YAML Information</a>
    """

    def representData(self, data: "Object") -> "Node":
        """
        Create a Node

        Arguments
        - data: the instance to represent

        Returns
        - Node to dump
        """
        ...



"""
Python module generated from Java source file java.util.zip.Checksum

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.util.zip import *
from typing import Any, Callable, Iterable, Tuple


class Checksum:
    """
    An interface representing a data checksum.

    Author(s)
    - David Connelly

    Since
    - 1.1
    """

    def update(self, b: int) -> None:
        """
        Updates the current checksum with the specified byte.

        Arguments
        - b: the byte to update the checksum with
        """
        ...


    def update(self, b: list[int]) -> None:
        """
        Updates the current checksum with the specified array of bytes.

        Arguments
        - b: the array of bytes to update the checksum with

        Raises
        - NullPointerException: if `b` is `null`

        Since
        - 9

        Unknown Tags
        - This default implementation is equal to calling
        `update(b, 0, b.length)`.
        """
        ...


    def update(self, b: list[int], off: int, len: int) -> None:
        """
        Updates the current checksum with the specified array of bytes.

        Arguments
        - b: the byte array to update the checksum with
        - off: the start offset of the data
        - len: the number of bytes to use for the update
        """
        ...


    def update(self, buffer: "ByteBuffer") -> None:
        """
        Updates the current checksum with the bytes from the specified buffer.
        
        The checksum is updated with the remaining bytes in the buffer, starting
        at the buffer's position. Upon return, the buffer's position will be
        updated to its limit; its limit will not have been changed.

        Arguments
        - buffer: the ByteBuffer to update the checksum with

        Raises
        - NullPointerException: if `buffer` is `null`

        Since
        - 9

        Unknown Tags
        - For best performance with DirectByteBuffer and other ByteBuffer
        implementations without a backing array implementers of this interface
        should override this method.
        - The default implementation has the following behavior.
        For ByteBuffers backed by an accessible byte array.
        ````update(buffer.array(),
               buffer.position() + buffer.arrayOffset(),
               buffer.remaining());````
        For ByteBuffers not backed by an accessible byte array.
        ````byte[] b = new byte[Math.min(buffer.remaining(), 4096)];
        while (buffer.hasRemaining()) {
            int length = Math.min(buffer.remaining(), b.length);
            buffer.get(b, 0, length);
            update(b, 0, length);`
        }```
        """
        ...


    def getValue(self) -> int:
        """
        Returns the current checksum value.

        Returns
        - the current checksum value
        """
        ...


    def reset(self) -> None:
        """
        Resets the checksum to its initial value.
        """
        ...



"""
Python module generated from Java source file org.bukkit.event.block.BlockPistonEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit import Material
from org.bukkit.block import Block
from org.bukkit.block import BlockFace
from org.bukkit.event import Cancellable
from org.bukkit.event.block import *
from typing import Any, Callable, Iterable, Tuple


class BlockPistonEvent(BlockEvent, Cancellable):
    """
    Called when a piston block is triggered
    """

    def __init__(self, block: "Block", direction: "BlockFace"):
        ...


    def isCancelled(self) -> bool:
        ...


    def setCancelled(self, cancelled: bool) -> None:
        ...


    def isSticky(self) -> bool:
        """
        Returns True if the Piston in the event is sticky.

        Returns
        - stickiness of the piston
        """
        ...


    def getDirection(self) -> "BlockFace":
        """
        Return the direction in which the piston will operate.

        Returns
        - direction of the piston
        """
        ...



"""
Python module generated from Java source file org.bukkit.loot.LootContext

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.base import Preconditions
from org.bukkit import Location
from org.bukkit.entity import Entity
from org.bukkit.entity import HumanEntity
from org.bukkit.loot import *
from typing import Any, Callable, Iterable, Tuple


class LootContext:
    """
    Represents additional information a LootTable can use to modify it's
    generated loot.
    """

    DEFAULT_LOOT_MODIFIER = -1


    def getLocation(self) -> "Location":
        """
        The Location to store where the loot will be generated.

        Returns
        - the Location of where the loot will be generated
        """
        ...


    def getLuck(self) -> float:
        """
        Represents the org.bukkit.potion.PotionEffectType.LUCK that an
        entity can have. The higher the value the better chance of receiving more
        loot.

        Returns
        - luck
        """
        ...


    def getLootingModifier(self) -> int:
        """
        Represents the
        org.bukkit.enchantments.Enchantment.LOOTING the
        .getKiller() entity has on their equipped item.
        
        This value is only set via
        LootContext.Builder.lootingModifier(int). If not set, the
        .getKiller() entity's looting level will be used instead.

        Returns
        - the looting level

        Deprecated
        - no longer functional
        """
        ...


    def getLootedEntity(self) -> "Entity":
        """
        Get the Entity that was killed. Can be null.

        Returns
        - the looted entity or null
        """
        ...


    def getKiller(self) -> "HumanEntity":
        """
        Get the HumanEntity who killed the .getLootedEntity().
        Can be null.

        Returns
        - the killer entity, or null.
        """
        ...


    class Builder:
        """
        Utility class to make building LootContext easier. The only
        required argument is Location with a valid (non-null)
        org.bukkit.World.
        """

        def __init__(self, location: "Location"):
            """
            Creates a new LootContext.Builder instance to facilitate easy
            creation of LootContexts.

            Arguments
            - location: the location the LootContext should use
            """
            ...


        def luck(self, luck: float) -> "Builder":
            """
            Set how much luck to have when generating loot.

            Arguments
            - luck: the luck level

            Returns
            - the Builder
            """
            ...


        def lootingModifier(self, modifier: int) -> "Builder":
            """
            Set the org.bukkit.enchantments.Enchantment.LOOTING
            level equivalent to use when generating loot. Values less than or
            equal to 0 will force the LootTable to only return a single
            org.bukkit.inventory.ItemStack per pool.

            Arguments
            - modifier: the looting level modifier

            Returns
            - the Builder

            Deprecated
            - no longer functional
            """
            ...


        def lootedEntity(self, lootedEntity: "Entity") -> "Builder":
            """
            The entity that was killed.

            Arguments
            - lootedEntity: the looted entity

            Returns
            - the Builder
            """
            ...


        def killer(self, killer: "HumanEntity") -> "Builder":
            """
            Set the org.bukkit.entity.HumanEntity that killed
            .getLootedEntity(). This entity will be used to get the
            looting level if .lootingModifier(int) is not set.

            Arguments
            - killer: the killer entity

            Returns
            - the Builder
            """
            ...


        def build(self) -> "LootContext":
            """
            Create a new LootContext instance using the supplied
            parameters.

            Returns
            - a new LootContext instance
            """
            ...



"""
Python module generated from Java source file org.bukkit.Raid

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from enum import Enum
from java.util import UUID
from org.bukkit import *
from org.bukkit.entity import Raider
from typing import Any, Callable, Iterable, Tuple


class Raid:
    """
    Represents a raid event.
    """

    def isStarted(self) -> bool:
        """
        Get whether this raid started.

        Returns
        - whether raid is started
        """
        ...


    def getActiveTicks(self) -> int:
        """
        Gets the amount of ticks this raid has existed.

        Returns
        - active ticks
        """
        ...


    def getBadOmenLevel(self) -> int:
        """
        Gets the Bad Omen level of this raid.

        Returns
        - Bad Omen level (between 0 and 5)
        """
        ...


    def setBadOmenLevel(self, badOmenLevel: int) -> None:
        """
        Sets the Bad Omen level.
        
        If the level is higher than 1, there will be an additional wave that as
        strong as the final wave.

        Arguments
        - badOmenLevel: new Bad Omen level (from 0-5)

        Raises
        - IllegalArgumentException: if invalid Bad Omen level
        """
        ...


    def getLocation(self) -> "Location":
        """
        Gets the center location where the raid occurs.

        Returns
        - location
        """
        ...


    def getStatus(self) -> "RaidStatus":
        """
        Gets the current status of the raid.
        
        Do not use this method to check if the raid has been started, call
        .isStarted() instead.

        Returns
        - Raids status
        """
        ...


    def getSpawnedGroups(self) -> int:
        """
        Gets the number of raider groups which have spawned.

        Returns
        - total spawned groups
        """
        ...


    def getTotalGroups(self) -> int:
        """
        Gets the number of raider groups which would spawn.
        
        This also includes the group which spawns in the additional wave (if
        present).

        Returns
        - total groups
        """
        ...


    def getTotalWaves(self) -> int:
        """
        Gets the number of waves in this raid (exclude the additional wave).

        Returns
        - number of waves
        """
        ...


    def getTotalHealth(self) -> float:
        """
        Gets the sum of all raider's health.

        Returns
        - total raiders health
        """
        ...


    def getHeroes(self) -> set["UUID"]:
        """
        Get the UUID of all heroes in this raid.

        Returns
        - a set of unique ids
        """
        ...


    def getRaiders(self) -> list["Raider"]:
        """
        Gets all remaining Raider in the present wave.

        Returns
        - a list of current raiders
        """
        ...


    class RaidStatus(Enum):
        """
        Represents the status of a Raid.
        """

        ONGOING = 0
        """
        The raid is in progress.
        """
        VICTORY = 1
        """
        The raid was beaten by heroes.
        """
        LOSS = 2
        """
        The village has fallen (i.e. all villagers died).
        """
        STOPPED = 3
        """
        The raid was terminated.
        """



"""
Python module generated from Java source file com.google.gson.stream.JsonReader

Java source file obtained from artifact gson version 2.11.0

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.gson import Gson
from com.google.gson import GsonBuilder
from com.google.gson import Strictness
from com.google.gson.internal import JsonReaderInternalAccess
from com.google.gson.internal import TroubleshootingGuide
from com.google.gson.internal.bind import JsonTreeReader
from com.google.gson.stream import *
from java.io import Closeable
from java.io import EOFException
from java.io import IOException
from java.io import Reader
from java.util import Arrays
from java.util import Objects
from typing import Any, Callable, Iterable, Tuple


class JsonReader(Closeable):
    """
    Reads a JSON (<a href="https://www.ietf.org/rfc/rfc8259.txt">RFC 8259</a>) encoded value as a
    stream of tokens. This stream includes both literal values (strings, numbers, booleans, and
    nulls) as well as the begin and end delimiters of objects and arrays. The tokens are traversed in
    depth-first order, the same order that they appear in the JSON document. Within JSON objects,
    name/value pairs are represented by a single token.
    
    <h2>Parsing JSON</h2>
    
    To create a recursive descent parser for your own JSON streams, first create an entry point
    method that creates a `JsonReader`.
    
    Next, create handler methods for each structure in your JSON text. You'll need a method for
    each object type and for each array type.
    
    
      - Within <strong>array handling</strong> methods, first call .beginArray to consume
          the array's opening bracket. Then create a while loop that accumulates values, terminating
          when .hasNext is False. Finally, read the array's closing bracket by calling .endArray.
      - Within <strong>object handling</strong> methods, first call .beginObject to consume
          the object's opening brace. Then create a while loop that assigns values to local variables
          based on their name. This loop should terminate when .hasNext is False. Finally,
          read the object's closing brace by calling .endObject.
    
    
    When a nested object or array is encountered, delegate to the corresponding handler method.
    
    When an unknown name is encountered, strict parsers should fail with an exception. Lenient
    parsers should call .skipValue() to recursively skip the value's nested tokens, which may
    otherwise conflict.
    
    If a value may be null, you should first check using .peek(). Null literals can be
    consumed using either .nextNull() or .skipValue().
    
    <h2>Configuration</h2>
    
    The behavior of this reader can be customized with the following methods:
    
    
      - .setStrictness(Strictness), the default is Strictness.LEGACY_STRICT
    
    
    The default configuration of `JsonReader` instances used internally by the Gson
    class differs, and can be adjusted with the various GsonBuilder methods.
    
    <h2>Example</h2>
    
    Suppose we'd like to parse a stream of messages such as the following:
    
    ````[
      {
        "id": 912345678901,
        "text": "How do I read a JSON stream in Java?",
        "geo": null,
        "user": {
          "name": "json_newb",
          "followers_count": 41`
      },
      {
        "id": 912345678902,
        "text": "@json_newb just use JsonReader!",
        "geo": [50.454722, -104.606667],
        "user": {
          "name": "jesse",
          "followers_count": 2
        }
      }
    ]
    }```
    
    This code implements the parser for the above structure:
    
    ````public List<Message> readJsonStream(InputStream in) throws IOException {
      JsonReader reader = new JsonReader(new InputStreamReader(in, "UTF-8"));
      try {
        return readMessagesArray(reader);` finally {
        reader.close();
      }
    }
    
    public List<Message> readMessagesArray(JsonReader reader) throws IOException {
      List<Message> messages = new ArrayList<>();
    
      reader.beginArray();
      while (reader.hasNext()) {
        messages.add(readMessage(reader));
      }
      reader.endArray();
      return messages;
    }
    
    public Message readMessage(JsonReader reader) throws IOException {
      long id = -1;
      String text = null;
      User user = null;
      List<Double> geo = null;
    
      reader.beginObject();
      while (reader.hasNext()) {
        String name = reader.nextName();
        if (name.equals("id")) {
          id = reader.nextLong();
        } else if (name.equals("text")) {
          text = reader.nextString();
        } else if (name.equals("geo") && reader.peek() != JsonToken.NULL) {
          geo = readDoublesArray(reader);
        } else if (name.equals("user")) {
          user = readUser(reader);
        } else {
          reader.skipValue();
        }
      }
      reader.endObject();
      return new Message(id, text, user, geo);
    }
    
    public List<Double> readDoublesArray(JsonReader reader) throws IOException {
      List<Double> doubles = new ArrayList<>();
    
      reader.beginArray();
      while (reader.hasNext()) {
        doubles.add(reader.nextDouble());
      }
      reader.endArray();
      return doubles;
    }
    
    public User readUser(JsonReader reader) throws IOException {
      String username = null;
      int followersCount = -1;
    
      reader.beginObject();
      while (reader.hasNext()) {
        String name = reader.nextName();
        if (name.equals("name")) {
          username = reader.nextString();
        } else if (name.equals("followers_count")) {
          followersCount = reader.nextInt();
        } else {
          reader.skipValue();
        }
      }
      reader.endObject();
      return new User(username, followersCount);
    }
    }```
    
    <h2>Number Handling</h2>
    
    This reader permits numeric values to be read as strings and string values to be read as numbers.
    For example, both elements of the JSON array `[1, "1"]` may be read using either .nextInt or .nextString. This behavior is intended to prevent lossy numeric conversions:
    double is JavaScript's only numeric type and very large values like `9007199254740993`
    cannot be represented exactly on that platform. To minimize precision loss, extremely large
    values should be written and read as strings in JSON.
    
    <h2 id="nonexecuteprefix">Non-Execute Prefix</h2>
    
    Web servers that serve private data using JSON may be vulnerable to <a
    href="http://en.wikipedia.org/wiki/JSON#Cross-site_request_forgery">Cross-site request
    forgery</a> attacks. In such an attack, a malicious site gains access to a private JSON file by
    executing it with an HTML `<script>` tag.
    
    Prefixing JSON files with `")]}'\n"` makes them non-executable by `<script>`
    tags, disarming the attack. Since the prefix is malformed JSON, strict parsing fails when it is
    encountered. This class permits the non-execute prefix when .setStrictness(Strictness) lenient parsing is enabled.
    
    Each `JsonReader` may be used to read a single JSON stream. Instances of this class are
    not thread safe.

    Author(s)
    - Jesse Wilson

    Since
    - 1.6
    """

    def __init__(self, in: "Reader"):
        """
        Creates a new instance that reads a JSON-encoded stream from `in`.
        """
        ...


    def setLenient(self, lenient: bool) -> None:
        """
        Sets the strictness of this reader.

        Arguments
        - lenient: whether this reader should be lenient. If True, the strictness is set to Strictness.LENIENT. If False, the strictness is set to Strictness.LEGACY_STRICT.

        See
        - .setStrictness(Strictness)

        Deprecated
        - Please use .setStrictness(Strictness) instead. `JsonReader.setLenient(True)` should be replaced by `JsonReader.setStrictness(Strictness.LENIENT)` and `JsonReader.setLenient(False)`
            should be replaced by `JsonReader.setStrictness(Strictness.LEGACY_STRICT)`.
            However, if you used `setLenient(False)` before, you might prefer Strictness.STRICT now instead.
        """
        ...


    def isLenient(self) -> bool:
        """
        Returns True if the Strictness of this reader is equal to Strictness.LENIENT.

        See
        - .getStrictness()
        """
        ...


    def setStrictness(self, strictness: "Strictness") -> None:
        """
        Configures how liberal this parser is in what it accepts.
        
        In Strictness.STRICT strict mode, the parser only accepts JSON in accordance
        with <a href="https://www.ietf.org/rfc/rfc8259.txt">RFC 8259</a>. In Strictness.LEGACY_STRICT legacy strict mode (the default), only JSON in accordance with the
        RFC 8259 is accepted, with a few exceptions denoted below for backwards compatibility reasons.
        In Strictness.LENIENT lenient mode, all sort of non-spec compliant JSON is
        accepted (see below).
        
        <dl>
          <dt>Strictness.STRICT
          <dd>In strict mode, only input compliant with RFC 8259 is accepted.
          <dt>Strictness.LEGACY_STRICT
          <dd>In legacy strict mode, the following departures from RFC 8259 are accepted:
              
                - JsonReader allows the literals `True`, `False` and `null` to have
                    any capitalization, for example `fAlSe` or `NULL`
                - JsonReader supports the escape sequence `\'`, representing a `'`
                    (single-quote)
                - JsonReader supports the escape sequence `\*LF*` (with `LF`
                    being the Unicode character `U+000A`), resulting in a `LF` within the
                    read JSON string
                - JsonReader allows unescaped control characters (`U+0000` through `U+001F`)
              
          <dt>Strictness.LENIENT
          <dd>In lenient mode, all input that is accepted in legacy strict mode is accepted in addition
              to the following departures from RFC 8259:
              
                - Streams that start with the <a href="#nonexecuteprefix">non-execute prefix</a>,
                    `")]`'\n"}
                - Streams that include multiple top-level values. With legacy strict or strict
                    parsing, each stream must contain exactly one top-level value.
                - Numbers may be Double.isNaN() NaNs or Double.isInfinite()
                    infinities represented by `NaN` and `(-)Infinity` respectively.
                - End of line comments starting with `//` or `.` and ending with a
                    newline character.
                - C-style comments starting with `/*` and ending with `*``/`. Such
                    comments may not be nested.
                - Names that are unquoted or `'single quoted'`.
                - Strings that are unquoted or `'single quoted'`.
                - Array elements separated by `;` instead of `,`.
                - Unnecessary array separators. These are interpreted as if null was the omitted
                    value.
                - Names and values separated by `=` or `=>` instead of `:`.
                - Name/value pairs separated by `;` instead of `,`.
              
        </dl>

        Arguments
        - strictness: the new strictness value of this reader. May not be `null`.

        See
        - .getStrictness()

        Since
        - 2.11.0
        """
        ...


    def getStrictness(self) -> "Strictness":
        """
        Returns the Strictness strictness of this reader.

        See
        - .setStrictness(Strictness)

        Since
        - 2.11.0
        """
        ...


    def beginArray(self) -> None:
        """
        Consumes the next token from the JSON stream and asserts that it is the beginning of a new
        array.

        Raises
        - IllegalStateException: if the next token is not the beginning of an array.
        """
        ...


    def endArray(self) -> None:
        """
        Consumes the next token from the JSON stream and asserts that it is the end of the current
        array.

        Raises
        - IllegalStateException: if the next token is not the end of an array.
        """
        ...


    def beginObject(self) -> None:
        """
        Consumes the next token from the JSON stream and asserts that it is the beginning of a new
        object.

        Raises
        - IllegalStateException: if the next token is not the beginning of an object.
        """
        ...


    def endObject(self) -> None:
        """
        Consumes the next token from the JSON stream and asserts that it is the end of the current
        object.

        Raises
        - IllegalStateException: if the next token is not the end of an object.
        """
        ...


    def hasNext(self) -> bool:
        """
        Returns True if the current array or object has another element.
        """
        ...


    def peek(self) -> "JsonToken":
        """
        Returns the type of the next token without consuming it.
        """
        ...


    def nextName(self) -> str:
        """
        Returns the next token, a JsonToken.NAME property name, and consumes it.

        Raises
        - IllegalStateException: if the next token is not a property name.
        """
        ...


    def nextString(self) -> str:
        """
        Returns the JsonToken.STRING string value of the next token, consuming it. If the next
        token is a number, this method will return its string form.

        Raises
        - IllegalStateException: if the next token is not a string.
        """
        ...


    def nextBoolean(self) -> bool:
        """
        Returns the JsonToken.BOOLEAN boolean value of the next token, consuming it.

        Raises
        - IllegalStateException: if the next token is not a boolean.
        """
        ...


    def nextNull(self) -> None:
        """
        Consumes the next token from the JSON stream and asserts that it is a literal null.

        Raises
        - IllegalStateException: if the next token is not a JSON null.
        """
        ...


    def nextDouble(self) -> float:
        """
        Returns the JsonToken.NUMBER double value of the next token, consuming it. If the next
        token is a string, this method will attempt to parse it as a double using Double.parseDouble(String).

        Raises
        - IllegalStateException: if the next token is neither a number nor a string.
        - NumberFormatException: if the next literal value cannot be parsed as a double.
        - MalformedJsonException: if the next literal value is NaN or Infinity and this reader is
            not .setStrictness(Strictness) lenient.
        """
        ...


    def nextLong(self) -> int:
        """
        Returns the JsonToken.NUMBER long value of the next token, consuming it. If the next
        token is a string, this method will attempt to parse it as a long. If the next token's numeric
        value cannot be exactly represented by a Java `long`, this method throws.

        Raises
        - IllegalStateException: if the next token is neither a number nor a string.
        - NumberFormatException: if the next literal value cannot be parsed as a number, or
            exactly represented as a long.
        """
        ...


    def nextInt(self) -> int:
        """
        Returns the JsonToken.NUMBER int value of the next token, consuming it. If the next
        token is a string, this method will attempt to parse it as an int. If the next token's numeric
        value cannot be exactly represented by a Java `int`, this method throws.

        Raises
        - IllegalStateException: if the next token is neither a number nor a string.
        - NumberFormatException: if the next literal value cannot be parsed as a number, or
            exactly represented as an int.
        """
        ...


    def close(self) -> None:
        """
        Closes this JSON reader and the underlying Reader.
        
        Using the JSON reader after it has been closed will throw an IllegalStateException
        in most cases.
        """
        ...


    def skipValue(self) -> None:
        """
        Skips the next value recursively. This method is intended for use when the JSON token stream
        contains unrecognized or unhandled values.
        
        The behavior depends on the type of the next JSON token:
        
        
          - Start of a JSON array or object: It and all of its nested values are skipped.
          - Primitive value (for example a JSON number): The primitive value is skipped.
          - Property name: Only the name but not the value of the property is skipped. `skipValue()` has to be called again to skip the property value as well.
          - End of a JSON array or object: Only this end token is skipped.
          - End of JSON document: Skipping has no effect, the next token continues to be the end of
              the document.
        """
        ...


    def toString(self) -> str:
        ...


    def getPath(self) -> str:
        """
        Returns a <a href="https://goessner.net/articles/JsonPath/">JSONPath</a> in *dot-notation*
        to the next (or current) location in the JSON document. That means:
        
        
          - For JSON arrays the path points to the index of the next element (even if there are no
              further elements).
          - For JSON objects the path points to the last property, or to the current property if its
              name has already been consumed.
        
        
        This method can be useful to add additional context to exception messages *before* a
        value is consumed, for example when the .peek() peeked token is unexpected.
        """
        ...


    def getPreviousPath(self) -> str:
        """
        Returns a <a href="https://goessner.net/articles/JsonPath/">JSONPath</a> in *dot-notation*
        to the previous (or current) location in the JSON document. That means:
        
        
          - For JSON arrays the path points to the index of the previous element.
              If no element has been consumed yet it uses the index 0 (even if there are no elements).
          - For JSON objects the path points to the last property, or to the current property if its
              name has already been consumed.
        
        
        This method can be useful to add additional context to exception messages *after* a
        value has been consumed.
        """
        ...



"""
Python module generated from Java source file org.bukkit.inventory.ItemFactory

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit import Color
from org.bukkit import Material
from org.bukkit import Server
from org.bukkit import World
from org.bukkit.enchantments import Enchantment
from org.bukkit.entity import Entity
from org.bukkit.entity import EntityType
from org.bukkit.inventory import *
from org.bukkit.inventory.meta import BookMeta
from org.bukkit.inventory.meta import ItemMeta
from org.bukkit.inventory.meta import SkullMeta
from typing import Any, Callable, Iterable, Tuple


class ItemFactory:
    """
    An instance of the ItemFactory can be obtained with Server.getItemFactory().
    
    The ItemFactory is solely responsible for creating item meta containers to
    apply on item stacks.
    """

    def getItemMeta(self, material: "Material") -> "ItemMeta":
        """
        This creates a new item meta for the material.

        Arguments
        - material: The material to consider as base for the meta

        Returns
        - a new ItemMeta that could be applied to an item stack of the
            specified material
        """
        ...


    def isApplicable(self, meta: "ItemMeta", stack: "ItemStack") -> bool:
        """
        This method checks the item meta to confirm that it is applicable (no
        data lost if applied) to the specified ItemStack.
        
        A SkullMeta would not be valid for a sword, but a normal ItemMeta from an enchanted dirt block would.

        Arguments
        - meta: Meta to check
        - stack: Item that meta will be applied to

        Returns
        - True if the meta can be applied without losing data, False
            otherwise

        Raises
        - IllegalArgumentException: if the meta was not created by this
            factory
        """
        ...


    def isApplicable(self, meta: "ItemMeta", material: "Material") -> bool:
        """
        This method checks the item meta to confirm that it is applicable (no
        data lost if applied) to the specified Material.
        
        A SkullMeta would not be valid for a sword, but a normal ItemMeta from an enchanted dirt block would.

        Arguments
        - meta: Meta to check
        - material: Material that meta will be applied to

        Returns
        - True if the meta can be applied without losing data, False
            otherwise

        Raises
        - IllegalArgumentException: if the meta was not created by this
            factory
        """
        ...


    def equals(self, meta1: "ItemMeta", meta2: "ItemMeta") -> bool:
        """
        This method is used to compare two item meta data objects.

        Arguments
        - meta1: First meta to compare, and may be null to indicate no data
        - meta2: Second meta to compare, and may be null to indicate no
            data

        Returns
        - False if one of the meta has data the other does not, otherwise
            True

        Raises
        - IllegalArgumentException: if either meta was not created by this
            factory
        """
        ...


    def asMetaFor(self, meta: "ItemMeta", stack: "ItemStack") -> "ItemMeta":
        """
        Returns an appropriate item meta for the specified stack.
        
        The item meta returned will always be a valid meta for a given
        ItemStack of the specified material. It may be a more or less specific
        meta, and could also be the same meta or meta type as the parameter.
        The item meta returned will also always be the most appropriate meta.
        
        Example, if a SkullMeta is being applied to a book, this method
        would return a BookMeta containing all information in the
        specified meta that is applicable to an ItemMeta, the highest
        common interface.

        Arguments
        - meta: the meta to convert
        - stack: the stack to convert the meta for

        Returns
        - An appropriate item meta for the specified item stack. No
            guarantees are made as to if a copy is returned. This will be null
            for a stack of air.

        Raises
        - IllegalArgumentException: if the specified meta was not created
            by this factory
        """
        ...


    def asMetaFor(self, meta: "ItemMeta", material: "Material") -> "ItemMeta":
        """
        Returns an appropriate item meta for the specified material.
        
        The item meta returned will always be a valid meta for a given
        ItemStack of the specified material. It may be a more or less specific
        meta, and could also be the same meta or meta type as the parameter.
        The item meta returned will also always be the most appropriate meta.
        
        Example, if a SkullMeta is being applied to a book, this method
        would return a BookMeta containing all information in the
        specified meta that is applicable to an ItemMeta, the highest
        common interface.

        Arguments
        - meta: the meta to convert
        - material: the material to convert the meta for

        Returns
        - An appropriate item meta for the specified item material. No
            guarantees are made as to if a copy is returned. This will be null for air.

        Raises
        - IllegalArgumentException: if the specified meta was not created
            by this factory
        """
        ...


    def getDefaultLeatherColor(self) -> "Color":
        """
        Returns the default color for all leather armor.

        Returns
        - the default color for leather armor
        """
        ...


    def createItemStack(self, input: str) -> "ItemStack":
        """
        Create a new ItemStack given the supplied input.
        
        The input should match the same input as expected by Minecraft's `/give`
        command. For example,
        ```"minecraft:diamond_sword[minecraft:enchantments={levels:{"minecraft:sharpness": 3}}]"```
        would yield an ItemStack of Material.DIAMOND_SWORD with an ItemMeta
        containing a level 3 Enchantment.SHARPNESS enchantment.

        Arguments
        - input: the item input string

        Returns
        - the created ItemStack

        Raises
        - IllegalArgumentException: if the input string was provided in an
        invalid or unsupported format
        """
        ...


    def getSpawnEgg(self, type: "EntityType") -> "Material":
        """
        Gets a Material representing the spawn egg for the provided
        EntityType. 
        Will return null for EntityTypes that do not have a corresponding spawn egg.

        Arguments
        - type: the entity type

        Returns
        - the Material of this EntityTypes spawn egg or null
        """
        ...


    def enchantItem(self, entity: "Entity", item: "ItemStack", level: int, allowTreasures: bool) -> "ItemStack":
        """
        Enchants the given item at the provided level.
        
        If an item that is air is passed through an error is thrown.

        Arguments
        - entity: the entity to use as a source of randomness
        - item: the item to enchant
        - level: the level to use, which is the level in the enchantment table
        - allowTreasures: allows treasure enchants, e.g. mending, if True.

        Returns
        - a new ItemStack containing the result of the Enchantment
        """
        ...


    def enchantItem(self, world: "World", item: "ItemStack", level: int, allowTreasures: bool) -> "ItemStack":
        """
        Enchants the given item at the provided level.
        
        If an item that is air is passed through an error is thrown.

        Arguments
        - world: the world to use as a source of randomness
        - item: the item to enchant
        - level: the level to use, which is the level in the enchantment table
        - allowTreasures: allow the treasure enchants, e.g. mending, if True.

        Returns
        - a new ItemStack containing the result of the Enchantment
        """
        ...


    def enchantItem(self, item: "ItemStack", level: int, allowTreasures: bool) -> "ItemStack":
        """
        Enchants the given item at the provided level.
        
        If an item that is air is passed through an error is thrown.

        Arguments
        - item: the item to enchant
        - level: the level to use, which is the level in the enchantment table
        - allowTreasures: allow treasure enchantments, e.g. mending, if True.

        Returns
        - a new ItemStack containing the result of the Enchantment
        """
        ...



from .PoissonSampling import PoissonSampling
from .UniformSampling import UniformSampling
from .StratifiedSampling import StratifiedSampling
from .Callback2d import Callback2d
from .Convolution import Convolution
from .SpiralSampling import SpiralSampling
from .BestCandidateSampling import BestCandidateSampling
from .Callback3d import Callback3d
from .Math import Math


"""
Python module generated from Java source file org.yaml.snakeyaml.representer.BaseRepresenter

Java source file obtained from artifact snakeyaml version 2.2

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.util import IdentityHashMap
from org.yaml.snakeyaml import DumperOptions
from org.yaml.snakeyaml.DumperOptions import FlowStyle
from org.yaml.snakeyaml.DumperOptions import ScalarStyle
from org.yaml.snakeyaml.introspector import PropertyUtils
from org.yaml.snakeyaml.nodes import AnchorNode
from org.yaml.snakeyaml.nodes import MappingNode
from org.yaml.snakeyaml.nodes import Node
from org.yaml.snakeyaml.nodes import NodeTuple
from org.yaml.snakeyaml.nodes import ScalarNode
from org.yaml.snakeyaml.nodes import SequenceNode
from org.yaml.snakeyaml.nodes import Tag
from org.yaml.snakeyaml.representer import *
from typing import Any, Callable, Iterable, Tuple


class BaseRepresenter:
    """
    Represent basic YAML structures: scalar, sequence, mapping
    """

    def represent(self, data: "Object") -> "Node":
        ...


    def setDefaultScalarStyle(self, defaultStyle: "ScalarStyle") -> None:
        ...


    def getDefaultScalarStyle(self) -> "ScalarStyle":
        """
        getter

        Returns
        - scala style
        """
        ...


    def setDefaultFlowStyle(self, defaultFlowStyle: "FlowStyle") -> None:
        ...


    def getDefaultFlowStyle(self) -> "FlowStyle":
        """
        getter

        Returns
        - current flow style
        """
        ...


    def setPropertyUtils(self, propertyUtils: "PropertyUtils") -> None:
        ...


    def getPropertyUtils(self) -> "PropertyUtils":
        """
        getter

        Returns
        - utils or create if null
        """
        ...


    def isExplicitPropertyUtils(self) -> bool:
        ...



"""
Python module generated from Java source file org.bukkit.block.SculkSensor

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.block import *
from typing import Any, Callable, Iterable, Tuple


class SculkSensor(TileState):
    """
    Represents a captured state of a sculk sensor
    """

    def getLastVibrationFrequency(self) -> int:
        """
        Gets the last vibration frequency of this sensor.
        
        Different activities detected by the sensor will produce different
        frequencies and dictate the output of connected comparators.

        Returns
        - frequency between 0-15.
        """
        ...


    def setLastVibrationFrequency(self, lastVibrationFrequency: int) -> None:
        """
        Sets the last vibration frequency of this sensor.
        
        Different activities detected by the sensor will produce different
        frequencies and dictate the output of connected comparators.

        Arguments
        - lastVibrationFrequency: frequency between 0-15.
        """
        ...
